# Copyright 2013 OpenStack Foundation
# Copyright 2015 Mirantis, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
import time
from unittest import mock

import ddt
from oslo_config import cfg

from manila.common import constants
from manila import context
from manila.db import api as db_api
from manila import exception
from manila.network.neutron import api as neutron_api
from manila.network.neutron import constants as neutron_constants
from manila.network.neutron import neutron_network_plugin as plugin
from manila import test
from manila.tests import utils as test_utils

CONF = cfg.CONF

fake_neutron_port = {
    "status": "ACTIVE",
    "allowed_address_pairs": [],
    "admin_state_up": True,
    "network_id": "test_net_id",
    "tenant_id": "fake_tenant_id",
    "extra_dhcp_opts": [],
    "device_owner": "test",
    "binding:capabilities": {"port_filter": True},
    "mac_address": "test_mac",
    "fixed_ips": [
        {"subnet_id": "test_subnet_id", "ip_address": "203.0.113.100"},
    ],
    "id": "test_port_id",
    "security_groups": ["fake_sec_group_id"],
    "device_id": "fake_device_id",
}

fake_neutron_network = {
    'admin_state_up': True,
    'availability_zone_hints': [],
    'availability_zones': ['nova'],
    'description': '',
    'id': 'fake net id',
    'ipv4_address_scope': None,
    'ipv6_address_scope': None,
    'name': 'test_neutron_network',
    'port_security_enabled': True,
    'provider:network_type': 'vxlan',
    'provider:physical_network': None,
    'provider:segmentation_id': 1234,
    'router:external': False,
    'shared': False,
    'status': 'ACTIVE',
    'subnets': ['fake subnet id',
                'fake subnet id 2'],
}

fake_ip_version = 4

fake_neutron_subnet = {
    'cidr': '10.0.0.0/24',
    'ip_version': fake_ip_version,
    'gateway_ip': '10.0.0.1',
}

fake_share_network_subnet = {
    'id': 'fake nw subnet id',
    'neutron_subnet_id': fake_neutron_network['subnets'][0],
    'neutron_net_id': fake_neutron_network['id'],
    'network_type': 'fake_network_type',
    'segmentation_id': 1234,
    'ip_version': 4,
    'cidr': 'fake_cidr',
    'gateway': 'fake_gateway',
    'mtu': 1509,
}

fake_share_network = {
    'id': 'fake nw info id',
    'project_id': 'fake project id',
    'status': 'test_subnet_status',
    'name': 'fake name',
    'description': 'fake description',
    'security_services': [],
    'subnets': [fake_share_network_subnet],
}

fake_share_server = {
    'id': 'fake nw info id',
    'status': 'test_server_status',
    'host': 'fake@host',
    'network_allocations': [],
    'shares': [],
}

fake_network_allocation = {
    'id': fake_neutron_port['id'],
    'share_server_id': fake_share_server['id'],
    'ip_address': fake_neutron_port['fixed_ips'][0]['ip_address'],
    'mac_address': fake_neutron_port['mac_address'],
    'status': constants.STATUS_ACTIVE,
    'label': 'user',
    'network_type': fake_share_network_subnet['network_type'],
    'segmentation_id': fake_share_network_subnet['segmentation_id'],
    'ip_version': fake_share_network_subnet['ip_version'],
    'cidr': fake_share_network_subnet['cidr'],
    'gateway': fake_share_network_subnet['gateway'],
    'mtu': 1509,
}

fake_nw_info = {
    'segments': [
        {
            'provider:network_type': 'vlan',
            'provider:physical_network': 'net1',
            'provider:segmentation_id': 3926,
        },
        {
            'provider:network_type': 'vxlan',
            'provider:physical_network': None,
            'provider:segmentation_id': 2000,
        },
    ],
    'mtu': 1509,
}

fake_neutron_network_multi = {
    'admin_state_up': True,
    'availability_zone_hints': [],
    'availability_zones': ['nova'],
    'description': '',
    'id': 'fake net id',
    'ipv4_address_scope': None,
    'ipv6_address_scope': None,
    'name': 'test_neutron_network',
    'port_security_enabled': True,
    'router:external': False,
    'shared': False,
    'status': 'ACTIVE',
    'subnets': ['fake subnet id',
                'fake subnet id 2'],
    'segments': fake_nw_info['segments'],
    'mtu': fake_nw_info['mtu'],
}

fake_share_network_multi = {
    'id': 'fake nw info id',
    'neutron_subnet_id': fake_neutron_network_multi['subnets'][0],
    'neutron_net_id': fake_neutron_network_multi['id'],
    'project_id': 'fake project id',
    'status': 'test_subnet_status',
    'name': 'fake name',
    'description': 'fake description',
    'security_services': [],
    'ip_version': None,
    'cidr': 'fake_cidr',
    'gateway': 'fake_gateway',
    'mtu': fake_neutron_network_multi['mtu'] - 1,
}

fake_network_allocation_multi = {
    'id': fake_neutron_port['id'],
    'share_server_id': fake_share_server['id'],
    'ip_address': fake_neutron_port['fixed_ips'][0]['ip_address'],
    'mac_address': fake_neutron_port['mac_address'],
    'status': constants.STATUS_ACTIVE,
    'label': 'user',
    'network_type': None,
    'segmentation_id': None,
    'ip_version': fake_neutron_subnet['ip_version'],
    'cidr': fake_neutron_subnet['cidr'],
    'gateway': fake_neutron_subnet['gateway_ip'],
    'mtu': fake_neutron_network_multi['mtu'],
}

fake_binding_profile = {
    'neutron_switch_id': 'fake switch id',
    'neutron_port_id': 'fake port id',
    'neutron_switch_info': 'fake switch info'
}


@ddt.ddt
class NeutronNetworkPluginTest(test.TestCase):

    def setUp(self):
        super(NeutronNetworkPluginTest, self).setUp()
        self.plugin = self._get_neutron_network_plugin_instance()
        self.plugin.db = db_api
        self.fake_context = context.RequestContext(user_id='fake user',
                                                   project_id='fake project',
                                                   is_admin=False)

    def _get_neutron_network_plugin_instance(self, config_data=None):
        if config_data is None:
            return plugin.NeutronNetworkPlugin()
        with test_utils.create_temp_config_with_opts(config_data):
            return plugin.NeutronNetworkPlugin()

    @mock.patch.object(db_api, 'network_allocation_create',
                       mock.Mock(return_values=fake_network_allocation))
    @mock.patch.object(db_api, 'share_network_get',
                       mock.Mock(return_value=fake_share_network))
    @mock.patch.object(db_api, 'share_server_get',
                       mock.Mock(return_value=fake_share_server))
    def test_allocate_network_one_allocation(self):
        has_provider_nw_ext = mock.patch.object(
            self.plugin, '_has_provider_network_extension').start()
        has_provider_nw_ext.return_value = True
        save_nw_data = mock.patch.object(self.plugin,
                                         '_save_neutron_network_data').start()
        save_subnet_data = mock.patch.object(
            self.plugin,
            '_save_neutron_subnet_data').start()

        with mock.patch.object(self.plugin.neutron_api, 'create_port',
                               mock.Mock(return_value=fake_neutron_port)):
            self.plugin.allocate_network(
                self.fake_context,
                fake_share_server,
                fake_share_network,
                fake_share_network_subnet,
                allocation_info={'count': 1})

            has_provider_nw_ext.assert_any_call()
            save_nw_data.assert_called_once_with(self.fake_context,
                                                 fake_share_network_subnet)
            save_subnet_data.assert_called_once_with(self.fake_context,
                                                     fake_share_network_subnet)
            self.plugin.neutron_api.create_port.assert_called_once_with(
                fake_share_network['project_id'],
                network_id=fake_share_network_subnet['neutron_net_id'],
                subnet_id=fake_share_network_subnet['neutron_subnet_id'],
                device_owner='manila:share',
                device_id=fake_share_network['id'])
            db_api.network_allocation_create.assert_called_once_with(
                self.fake_context,
                fake_network_allocation)

            has_provider_nw_ext.stop()
            save_nw_data.stop()
            save_subnet_data.stop()

    @mock.patch.object(db_api, 'network_allocation_create',
                       mock.Mock(return_values=fake_network_allocation))
    @mock.patch.object(db_api, 'share_network_get',
                       mock.Mock(return_value=fake_share_network))
    @mock.patch.object(db_api, 'share_server_get',
                       mock.Mock(return_value=fake_share_server))
    def test_allocate_network_two_allocation(self):
        has_provider_nw_ext = mock.patch.object(
            self.plugin, '_has_provider_network_extension').start()
        has_provider_nw_ext.return_value = True
        save_nw_data = mock.patch.object(self.plugin,
                                         '_save_neutron_network_data').start()
        save_subnet_data = mock.patch.object(
            self.plugin,
            '_save_neutron_subnet_data').start()

        with mock.patch.object(self.plugin.neutron_api, 'create_port',
                               mock.Mock(return_value=fake_neutron_port)):
            self.plugin.allocate_network(
                self.fake_context, fake_share_server, fake_share_network,
                fake_share_network_subnet, count=2)

            neutron_api_calls = [
                mock.call(
                    fake_share_network['project_id'],
                    network_id=fake_share_network_subnet['neutron_net_id'],
                    subnet_id=fake_share_network_subnet['neutron_subnet_id'],
                    device_owner='manila:share',
                    device_id=fake_share_network['id']),
                mock.call(
                    fake_share_network['project_id'],
                    network_id=fake_share_network_subnet['neutron_net_id'],
                    subnet_id=fake_share_network_subnet['neutron_subnet_id'],
                    device_owner='manila:share',
                    device_id=fake_share_network['id']),
            ]
            db_api_calls = [
                mock.call(self.fake_context, fake_network_allocation),
                mock.call(self.fake_context, fake_network_allocation)
            ]
            self.plugin.neutron_api.create_port.assert_has_calls(
                neutron_api_calls)
            db_api.network_allocation_create.assert_has_calls(db_api_calls)

            has_provider_nw_ext.stop()
            save_nw_data.stop()
            save_subnet_data.stop()

    @mock.patch.object(db_api, 'share_network_update', mock.Mock())
    def test_allocate_network_create_port_exception(self):
        has_provider_nw_ext = mock.patch.object(
            self.plugin, '_has_provider_network_extension').start()
        has_provider_nw_ext.return_value = True
        save_nw_data = mock.patch.object(self.plugin,
                                         '_save_neutron_network_data').start()
        save_subnet_data = mock.patch.object(
            self.plugin,
            '_save_neutron_subnet_data').start()
        create_port = mock.patch.object(self.plugin.neutron_api,
                                        'create_port').start()
        create_port.side_effect = exception.NetworkException

        self.assertRaises(exception.NetworkException,
                          self.plugin.allocate_network,
                          self.fake_context,
                          fake_share_server,
                          fake_share_network)

        has_provider_nw_ext.stop()
        save_nw_data.stop()
        save_subnet_data.stop()
        create_port.stop()

    def _setup_manage_network_allocations(self):

        allocations = ['192.168.0.11', '192.168.0.12', 'fd12::2000']

        neutron_ports = [
            copy.deepcopy(fake_neutron_port), copy.deepcopy(fake_neutron_port),
            copy.deepcopy(fake_neutron_port), copy.deepcopy(fake_neutron_port),
        ]

        neutron_ports[0]['fixed_ips'][0]['ip_address'] = '192.168.0.10'
        neutron_ports[0]['id'] = 'fake_port_id_0'
        neutron_ports[1]['fixed_ips'][0]['ip_address'] = '192.168.0.11'
        neutron_ports[1]['id'] = 'fake_port_id_1'
        neutron_ports[2]['fixed_ips'][0]['ip_address'] = '192.168.0.12'
        neutron_ports[2]['id'] = 'fake_port_id_2'
        neutron_ports[3]['fixed_ips'][0]['ip_address'] = '192.168.0.13'
        neutron_ports[3]['id'] = 'fake_port_id_3'

        self.mock_object(self.plugin, '_verify_share_network_subnet')
        self.mock_object(self.plugin, '_store_neutron_net_info')

        self.mock_object(self.plugin.neutron_api, 'list_ports',
                         mock.Mock(return_value=neutron_ports))

        return neutron_ports, allocations

    @ddt.data({}, exception.NotFound)
    def test_manage_network_allocations_create_update(self, side_effect):

        neutron_ports, allocations = self._setup_manage_network_allocations()

        self.mock_object(db_api, 'network_allocation_get',
                         mock.Mock(
                             side_effect=[exception.NotFound, side_effect,
                                          exception.NotFound, side_effect]))
        if side_effect:
            self.mock_object(db_api, 'network_allocation_create')
        else:
            self.mock_object(db_api, 'network_allocation_update')

        result = self.plugin.manage_network_allocations(
            self.fake_context, allocations, fake_share_server,
            share_network_subnet=fake_share_network_subnet)

        self.assertEqual(['fd12::2000'], result)

        self.plugin.neutron_api.list_ports.assert_called_once_with(
            network_id=fake_share_network_subnet['neutron_net_id'],
            device_owner='manila:share',
            fixed_ips='subnet_id=' +
                      fake_share_network_subnet['neutron_subnet_id'])

        db_api.network_allocation_get.assert_has_calls([
            mock.call(self.fake_context, 'fake_port_id_1', read_deleted=False),
            mock.call(self.fake_context, 'fake_port_id_1', read_deleted=True),
            mock.call(self.fake_context, 'fake_port_id_2', read_deleted=False),
            mock.call(self.fake_context, 'fake_port_id_2', read_deleted=True),
        ])

        port_dict_list = [{
            'share_server_id': fake_share_server['id'],
            'ip_address': x,
            'gateway': fake_share_network_subnet['gateway'],
            'mac_address': fake_neutron_port['mac_address'],
            'status': constants.STATUS_ACTIVE,
            'label': 'user',
            'network_type': fake_share_network_subnet['network_type'],
            'segmentation_id': fake_share_network_subnet['segmentation_id'],
            'ip_version': fake_share_network_subnet['ip_version'],
            'cidr': fake_share_network_subnet['cidr'],
            'mtu': fake_share_network_subnet['mtu'],
        } for x in ['192.168.0.11', '192.168.0.12']]

        if side_effect:
            port_dict_list[0]['id'] = 'fake_port_id_1'
            port_dict_list[1]['id'] = 'fake_port_id_2'
            db_api.network_allocation_create.assert_has_calls([
                mock.call(self.fake_context, port_dict_list[0]),
                mock.call(self.fake_context, port_dict_list[1])
            ])
        else:
            for x in port_dict_list:
                x['deleted_at'] = None
                x['deleted'] = 'False'

            db_api.network_allocation_update.assert_has_calls([
                mock.call(self.fake_context, 'fake_port_id_1',
                          port_dict_list[0], read_deleted=True),
                mock.call(self.fake_context, 'fake_port_id_2',
                          port_dict_list[1], read_deleted=True)
            ])

        self.plugin._verify_share_network_subnet.assert_called_once_with(
            fake_share_server['id'], fake_share_network_subnet)

        self.plugin._store_neutron_net_info(
            self.fake_context, fake_share_network_subnet)

    def test__get_ports_respective_to_ips_multiple_fixed_ips(self):
        self.mock_object(plugin.LOG, 'warning')

        allocations = ['192.168.0.10', '192.168.0.11', '192.168.0.12']

        neutron_ports = [
            copy.deepcopy(fake_neutron_port), copy.deepcopy(fake_neutron_port),
        ]

        neutron_ports[0]['fixed_ips'][0]['ip_address'] = '192.168.0.10'
        neutron_ports[0]['id'] = 'fake_port_id_0'
        neutron_ports[0]['fixed_ips'].append({'ip_address': '192.168.0.11',
                                              'subnet_id': 'test_subnet_id'})
        neutron_ports[1]['fixed_ips'][0]['ip_address'] = '192.168.0.12'
        neutron_ports[1]['id'] = 'fake_port_id_2'

        expected = [{'port': neutron_ports[0], 'allocation': '192.168.0.10'},
                    {'port': neutron_ports[1], 'allocation': '192.168.0.12'}]

        result = self.plugin._get_ports_respective_to_ips(allocations,
                                                          neutron_ports)

        self.assertEqual(expected, result)

        self.assertIs(True, plugin.LOG.warning.called)

    def test_manage_network_allocations_exception(self):

        neutron_ports, allocations = self._setup_manage_network_allocations()

        fake_allocation = {
            'id': 'fake_port_id',
            'share_server_id': 'fake_server_id'
        }

        self.mock_object(db_api, 'network_allocation_get',
                         mock.Mock(return_value=fake_allocation))

        self.assertRaises(
            exception.ManageShareServerError,
            self.plugin.manage_network_allocations, self.fake_context,
            allocations, fake_share_server, fake_share_network,
            fake_share_network_subnet)

        db_api.network_allocation_get.assert_called_once_with(
            self.fake_context, 'fake_port_id_1', read_deleted=False)

    def test_unmanage_network_allocations(self):

        neutron_ports = [
            copy.deepcopy(fake_neutron_port), copy.deepcopy(fake_neutron_port),
        ]

        neutron_ports[0]['id'] = 'fake_port_id_0'
        neutron_ports[1]['id'] = 'fake_port_id_1'

        get_mock = self.mock_object(
            db_api, 'network_allocations_get_for_share_server',
            mock.Mock(return_value=neutron_ports))

        self.mock_object(db_api, 'network_allocation_delete')

        self.plugin.unmanage_network_allocations(
            self.fake_context, fake_share_server['id'])

        get_mock.assert_called_once_with(
            self.fake_context, fake_share_server['id'])

        db_api.network_allocation_delete.assert_has_calls([
            mock.call(self.fake_context, 'fake_port_id_0'),
            mock.call(self.fake_context, 'fake_port_id_1')
        ])

    @mock.patch.object(db_api, 'network_allocation_delete', mock.Mock())
    @mock.patch.object(db_api, 'share_network_update', mock.Mock())
    @mock.patch.object(db_api, 'network_allocations_get_for_share_server',
                       mock.Mock(return_value=[fake_network_allocation]))
    def test_deallocate_network_nominal(self):
        share_srv = {'id': fake_share_server['id']}
        share_srv['network_allocations'] = [fake_network_allocation]

        with mock.patch.object(self.plugin.neutron_api, 'delete_port',
                               mock.Mock()):
            self.plugin.deallocate_network(self.fake_context, share_srv)
            self.plugin.neutron_api.delete_port.assert_called_once_with(
                fake_network_allocation['id'])
            db_api.network_allocation_delete.assert_called_once_with(
                self.fake_context,
                fake_network_allocation['id'])

    @mock.patch.object(db_api, 'share_network_update',
                       mock.Mock(return_value=fake_share_network))
    @mock.patch.object(db_api, 'network_allocation_update', mock.Mock())
    @mock.patch.object(db_api, 'network_allocations_get_for_share_server',
                       mock.Mock(return_value=[fake_network_allocation]))
    def test_deallocate_network_neutron_api_exception(self):
        share_srv = {'id': fake_share_server['id']}
        share_srv['network_allocations'] = [fake_network_allocation]

        delete_port = mock.patch.object(self.plugin.neutron_api,
                                        'delete_port').start()
        delete_port.side_effect = exception.NetworkException

        self.assertRaises(exception.NetworkException,
                          self.plugin.deallocate_network,
                          self.fake_context,
                          share_srv)
        db_api.network_allocation_update.assert_called_once_with(
            self.fake_context,
            fake_network_allocation['id'],
            {'status': constants.STATUS_ERROR})
        delete_port.stop()

    @mock.patch.object(db_api, 'share_network_subnet_update', mock.Mock())
    def test_save_neutron_network_data(self):
        neutron_nw_info = {
            'provider:network_type': 'vlan',
            'provider:segmentation_id': 1000,
            'mtu': 1509,
        }
        share_nw_update_dict = {
            'network_type': 'vlan',
            'segmentation_id': 1000,
            'mtu': 1509,
        }

        with mock.patch.object(self.plugin.neutron_api,
                               'get_network',
                               mock.Mock(return_value=neutron_nw_info)):
            self.plugin._save_neutron_network_data(self.fake_context,
                                                   fake_share_network_subnet)

            self.plugin.neutron_api.get_network.assert_called_once_with(
                fake_share_network_subnet['neutron_net_id'])
            self.plugin.db.share_network_subnet_update.assert_called_once_with(
                self.fake_context,
                fake_share_network_subnet['id'],
                share_nw_update_dict)

    @mock.patch.object(db_api, 'share_network_subnet_update', mock.Mock())
    def test_save_neutron_network_data_multi_segment(self):
        share_nw_update_dict = {
            'network_type': 'vlan',
            'segmentation_id': 3926,
            'mtu': 1509
        }
        config_data = {
            'DEFAULT': {
                'neutron_physical_net_name': 'net1',
            }
        }

        self.mock_object(self.plugin.neutron_api, 'get_network')
        self.plugin.neutron_api.get_network.return_value = fake_nw_info

        with test_utils.create_temp_config_with_opts(config_data):
            self.plugin._save_neutron_network_data(self.fake_context,
                                                   fake_share_network_subnet)

        self.plugin.neutron_api.get_network.assert_called_once_with(
            fake_share_network_subnet['neutron_net_id'])
        self.plugin.db.share_network_subnet_update.assert_called_once_with(
            self.fake_context,
            fake_share_network_subnet['id'],
            share_nw_update_dict)

    @mock.patch.object(db_api, 'share_network_update', mock.Mock())
    def test_save_neutron_network_data_multi_segment_without_ident(self):
        config_data = {
            'DEFAULT': {
                'neutron_physical_net_name': 'net100',
            }
        }

        self.mock_object(self.plugin.neutron_api, 'get_network')
        self.plugin.neutron_api.get_network.return_value = fake_nw_info

        with test_utils.create_temp_config_with_opts(config_data):
            self.assertRaises(exception.NetworkBadConfigurationException,
                              self.plugin._save_neutron_network_data,
                              self.fake_context, fake_share_network_subnet)

    @mock.patch.object(db_api, 'share_network_update', mock.Mock())
    def test_save_neutron_network_data_multi_segment_without_cfg(self):
        self.mock_object(self.plugin.neutron_api, 'get_network')
        self.plugin.neutron_api.get_network.return_value = fake_nw_info

        self.assertRaises(exception.NetworkBadConfigurationException,
                          self.plugin._save_neutron_network_data,
                          self.fake_context, fake_share_network_subnet)

    @mock.patch.object(db_api, 'share_network_subnet_update', mock.Mock())
    def test_save_neutron_subnet_data(self):
        neutron_subnet_info = fake_neutron_subnet
        subnet_value = {
            'cidr': '10.0.0.0/24',
            'ip_version': 4,
            'gateway': '10.0.0.1',
        }

        with mock.patch.object(self.plugin.neutron_api,
                               'get_subnet',
                               mock.Mock(return_value=neutron_subnet_info)):
            self.plugin._save_neutron_subnet_data(self.fake_context,
                                                  fake_share_network_subnet)

            self.plugin.neutron_api.get_subnet.assert_called_once_with(
                fake_share_network_subnet['neutron_subnet_id'])
            self.plugin.db.share_network_subnet_update.assert_called_once_with(
                self.fake_context,
                fake_share_network_subnet['id'],
                subnet_value)

    def test_has_network_provider_extension_true(self):
        extensions = {neutron_constants.PROVIDER_NW_EXT: {}}
        with mock.patch.object(self.plugin.neutron_api,
                               'list_extensions',
                               mock.Mock(return_value=extensions)):
            result = self.plugin._has_provider_network_extension()

            self.plugin.neutron_api.list_extensions.assert_any_call()
            self.assertTrue(result)

    def test_has_network_provider_extension_false(self):
        with mock.patch.object(self.plugin.neutron_api,
                               'list_extensions',
                               mock.Mock(return_value={})):
            result = self.plugin._has_provider_network_extension()

            self.plugin.neutron_api.list_extensions.assert_any_call()
            self.assertFalse(result)


@ddt.ddt
class NeutronSingleNetworkPluginTest(test.TestCase):

    def setUp(self):
        super(NeutronSingleNetworkPluginTest, self).setUp()
        self.context = 'fake_context'

    def test_init_valid(self):
        fake_net_id = 'fake_net_id'
        fake_subnet_id = 'fake_subnet_id'
        config_data = {
            'DEFAULT': {
                'neutron_net_id': fake_net_id,
                'neutron_subnet_id': fake_subnet_id,
            }
        }
        fake_net = {'subnets': ['fake1', 'fake2', fake_subnet_id]}
        self.mock_object(
            neutron_api.API, 'get_network', mock.Mock(return_value=fake_net))

        with test_utils.create_temp_config_with_opts(config_data):
            instance = plugin.NeutronSingleNetworkPlugin()

            self.assertEqual(fake_net_id, instance.net)
            self.assertEqual(fake_subnet_id, instance.subnet)
            neutron_api.API.get_network.assert_called_once_with(fake_net_id)

    @ddt.data(
        {'net': None, 'subnet': None},
        {'net': 'fake_net_id', 'subnet': None},
        {'net': None, 'subnet': 'fake_subnet_id'})
    @ddt.unpack
    def test_init_invalid(self, net, subnet):
        config_data = dict()
        # Simulate absence of set values
        if net:
            config_data['neutron_net_id'] = net
        if subnet:
            config_data['neutron_subnet_id'] = subnet
        config_data = dict(DEFAULT=config_data)

        with test_utils.create_temp_config_with_opts(config_data):
            self.assertRaises(
                exception.NetworkBadConfigurationException,
                plugin.NeutronSingleNetworkPlugin)

    @ddt.data({}, {'subnets': []}, {'subnets': ['different_foo_subnet']})
    def test_init_subnet_does_not_belong_to_net(self, fake_net):
        fake_net_id = 'fake_net_id'
        config_data = {
            'DEFAULT': {
                'neutron_net_id': fake_net_id,
                'neutron_subnet_id': 'fake_subnet_id',
            }
        }
        self.mock_object(
            neutron_api.API, 'get_network', mock.Mock(return_value=fake_net))

        with test_utils.create_temp_config_with_opts(config_data):
            self.assertRaises(
                exception.NetworkBadConfigurationException,
                plugin.NeutronSingleNetworkPlugin)
            neutron_api.API.get_network.assert_called_once_with(fake_net_id)

    def _get_neutron_network_plugin_instance(
            self, config_data=None, label=None):
        if not config_data:
            fake_subnet_id = 'fake_subnet_id'
            config_data = {
                'DEFAULT': {
                    'neutron_net_id': 'fake_net_id',
                    'neutron_subnet_id': fake_subnet_id,
                }
            }
            fake_net = {'subnets': [fake_subnet_id]}
            self.mock_object(
                neutron_api.API, 'get_network',
                mock.Mock(return_value=fake_net))
        with test_utils.create_temp_config_with_opts(config_data):
            instance = plugin.NeutronSingleNetworkPlugin(label=label)
            return instance

    def test___update_share_network_net_data_same_values(self):
        instance = self._get_neutron_network_plugin_instance()
        share_network = {
            'neutron_net_id': instance.net,
            'neutron_subnet_id': instance.subnet,
        }

        result = instance._update_share_network_net_data(
            self.context, share_network)

        self.assertEqual(share_network, result)

    def test___update_share_network_net_data_different_values_empty(self):
        instance = self._get_neutron_network_plugin_instance()
        share_network_input = {
            'id': 'fake_share_network_id',
        }
        share_network_result = {
            'neutron_net_id': instance.net,
            'neutron_subnet_id': instance.subnet,
        }
        self.mock_object(
            instance.db, 'share_network_subnet_update',
            mock.Mock(return_value='foo'))

        instance._update_share_network_net_data(
            self.context, share_network_input)

        instance.db.share_network_subnet_update.assert_called_once_with(
            self.context, share_network_input['id'], share_network_result)

    @ddt.data(
        {'n': 'fake_net_id', 's': 'bar'},
        {'n': 'foo', 's': 'fake_subnet_id'})
    @ddt.unpack
    def test___update_share_network_net_data_different_values(self, n, s):
        instance = self._get_neutron_network_plugin_instance()
        share_network = {
            'id': 'fake_share_network_id',
            'neutron_net_id': n,
            'neutron_subnet_id': s,
        }
        self.mock_object(
            instance.db, 'share_network_update',
            mock.Mock(return_value=share_network))

        self.assertRaises(
            exception.NetworkBadConfigurationException,
            instance._update_share_network_net_data,
            self.context, share_network)
        self.assertFalse(instance.db.share_network_update.called)

    def test_allocate_network(self):
        self.mock_object(plugin.NeutronNetworkPlugin, 'allocate_network')
        plugin.NeutronNetworkPlugin.allocate_network.return_value = [
            fake_neutron_port, fake_neutron_port]
        instance = self._get_neutron_network_plugin_instance()
        share_server = 'fake_share_server'
        share_network = {'id': 'fake_share_network'}
        share_network_subnet = {'id': 'fake_share_network_subnet'}
        share_network_subnet_upd = {'id': 'updated_fake_share_network_subnet'}
        count = 2
        device_owner = 'fake_device_owner'
        self.mock_object(
            instance, '_update_share_network_net_data',
            mock.Mock(return_value=share_network_subnet_upd))

        instance.allocate_network(
            self.context, share_server, share_network, share_network_subnet,
            count=count, device_owner=device_owner)

        instance._update_share_network_net_data.assert_called_once_with(
            self.context, share_network_subnet)
        plugin.NeutronNetworkPlugin.allocate_network.assert_called_once_with(
            self.context, share_server, share_network,
            share_network_subnet_upd, count=count, device_owner=device_owner)

    def test_manage_network_allocations(self):
        allocations = ['192.168.10.10', 'fd12::2000']
        instance = self._get_neutron_network_plugin_instance()
        parent = self.mock_object(
            plugin.NeutronNetworkPlugin, 'manage_network_allocations',
            mock.Mock(return_value=['fd12::2000']))
        self.mock_object(
            instance, '_update_share_network_net_data',
            mock.Mock(return_value=fake_share_network_subnet))

        result = instance.manage_network_allocations(
            self.context, allocations, fake_share_server, fake_share_network,
            fake_share_network_subnet)

        self.assertEqual(['fd12::2000'], result)

        instance._update_share_network_net_data.assert_called_once_with(
            self.context, fake_share_network_subnet)

        parent.assert_called_once_with(
            self.context, allocations, fake_share_server, fake_share_network,
            fake_share_network_subnet)

    def test_manage_network_allocations_admin(self):
        allocations = ['192.168.10.10', 'fd12::2000']
        instance = self._get_neutron_network_plugin_instance(label='admin')
        parent = self.mock_object(
            plugin.NeutronNetworkPlugin, 'manage_network_allocations',
            mock.Mock(return_value=['fd12::2000']))

        share_network_dict = {
            'project_id': instance.neutron_api.admin_project_id,
            'neutron_net_id': 'fake_net_id',
            'neutron_subnet_id': 'fake_subnet_id',
        }

        result = instance.manage_network_allocations(
            self.context, allocations, fake_share_server,
            share_network_subnet=share_network_dict)

        self.assertEqual(['fd12::2000'], result)

        parent.assert_called_once_with(
            self.context, allocations, fake_share_server, None,
            share_network_dict)


@ddt.ddt
class NeutronBindNetworkPluginTest(test.TestCase):
    def setUp(self):
        super(NeutronBindNetworkPluginTest, self).setUp()
        self.fake_context = context.RequestContext(user_id='fake user',
                                                   project_id='fake project',
                                                   is_admin=False)
        self.has_binding_ext_mock = self.mock_object(
            neutron_api.API, '_has_port_binding_extension')
        self.has_binding_ext_mock.return_value = True
        self.bind_plugin = self._get_neutron_network_plugin_instance()
        self.bind_plugin.db = db_api
        self.sleep_mock = self.mock_object(time, 'sleep')
        self.fake_share_network_multi = dict(fake_share_network_multi)

    def _get_neutron_network_plugin_instance(self, config_data=None):
        if config_data is None:
            return plugin.NeutronBindNetworkPlugin()
        with test_utils.create_temp_config_with_opts(config_data):
            return plugin.NeutronBindNetworkPlugin()

    def test_wait_for_bind(self):
        self.mock_object(self.bind_plugin.neutron_api, 'show_port')
        self.bind_plugin.neutron_api.show_port.return_value = fake_neutron_port

        self.bind_plugin._wait_for_ports_bind([fake_neutron_port],
                                              fake_share_server)

        self.bind_plugin.neutron_api.show_port.assert_called_once_with(
            fake_neutron_port['id'])
        self.sleep_mock.assert_not_called()

    def test_wait_for_bind_error(self):
        fake_neut_port = copy.copy(fake_neutron_port)
        fake_neut_port['status'] = 'ERROR'
        self.mock_object(self.bind_plugin.neutron_api, 'show_port')
        self.bind_plugin.neutron_api.show_port.return_value = fake_neut_port

        self.assertRaises(exception.NetworkException,
                          self.bind_plugin._wait_for_ports_bind,
                          [fake_neut_port, fake_neut_port],
                          fake_share_server)

        self.bind_plugin.neutron_api.show_port.assert_called_once_with(
            fake_neutron_port['id'])
        self.sleep_mock.assert_not_called()

    @ddt.data(('DOWN', 'ACTIVE'), ('DOWN', 'DOWN'), ('ACTIVE', 'DOWN'))
    def test_wait_for_bind_two_ports_no_bind(self, state):
        fake_neut_port1 = copy.copy(fake_neutron_port)
        fake_neut_port1['status'] = state[0]
        fake_neut_port2 = copy.copy(fake_neutron_port)
        fake_neut_port2['status'] = state[1]
        self.mock_object(self.bind_plugin.neutron_api, 'show_port')
        self.bind_plugin.neutron_api.show_port.side_effect = (
            [fake_neut_port1, fake_neut_port2] * 20)

        self.assertRaises(exception.NetworkBindException,
                          self.bind_plugin._wait_for_ports_bind,
                          [fake_neut_port1, fake_neut_port2],
                          fake_share_server)

    @mock.patch.object(db_api, 'share_network_get',
                       mock.Mock(return_value=fake_share_network))
    @mock.patch.object(db_api, 'share_server_get',
                       mock.Mock(return_value=fake_share_server))
    def test_allocate_network_one_allocation(self):
        self.mock_object(self.bind_plugin, '_has_provider_network_extension')
        self.bind_plugin._has_provider_network_extension.return_value = True
        save_nw_data = self.mock_object(self.bind_plugin,
                                        '_save_neutron_network_data')
        save_subnet_data = self.mock_object(self.bind_plugin,
                                            '_save_neutron_subnet_data')
        self.mock_object(self.bind_plugin, '_wait_for_ports_bind')
        neutron_host_id_opts = plugin.neutron_bind_network_plugin_opts[1]
        self.mock_object(neutron_host_id_opts, 'default')
        neutron_host_id_opts.default = 'foohost1'
        self.mock_object(db_api, 'network_allocation_create')
        db_api.network_allocation_create.return_value = fake_network_allocation
        self.mock_object(self.bind_plugin.neutron_api, 'get_network')
        self.bind_plugin.neutron_api.get_network.return_value = (
            fake_neutron_network)

        with mock.patch.object(self.bind_plugin.neutron_api, 'create_port',
                               mock.Mock(return_value=fake_neutron_port)):
            self.bind_plugin.allocate_network(
                self.fake_context,
                fake_share_server,
                fake_share_network,
                fake_share_network_subnet,
                allocation_info={'count': 1})

            self.bind_plugin._has_provider_network_extension.assert_any_call()
            save_nw_data.assert_called_once_with(self.fake_context,
                                                 fake_share_network_subnet)
            save_subnet_data.assert_called_once_with(self.fake_context,
                                                     fake_share_network_subnet)
            expected_kwargs = {
                'binding:vnic_type': 'baremetal',
                'host_id': 'foohost1',
                'network_id': fake_share_network_subnet['neutron_net_id'],
                'subnet_id': fake_share_network_subnet['neutron_subnet_id'],
                'device_owner': 'manila:share',
                'device_id': fake_share_network['id'],
            }
            self.bind_plugin.neutron_api.create_port.assert_called_once_with(
                fake_share_network['project_id'], **expected_kwargs)
            db_api.network_allocation_create.assert_called_once_with(
                self.fake_context,
                fake_network_allocation)
            self.bind_plugin._wait_for_ports_bind.assert_called_once_with(
                [db_api.network_allocation_create(
                    self.fake_context, fake_network_allocation)],
                fake_share_server)

    @mock.patch.object(db_api, 'network_allocation_create',
                       mock.Mock(return_values=fake_network_allocation_multi))
    @mock.patch.object(db_api, 'share_network_get',
                       mock.Mock(return_value=fake_share_network_multi))
    @mock.patch.object(db_api, 'share_server_get',
                       mock.Mock(return_value=fake_share_server))
    def test_allocate_network_multi_segment(self):
        network_allocation_update_data = {
            'network_type':
                fake_nw_info['segments'][0]['provider:network_type'],
            'segmentation_id':
                fake_nw_info['segments'][0]['provider:segmentation_id'],
        }
        network_update_data = dict(network_allocation_update_data)
        network_update_data['mtu'] = fake_nw_info['mtu']
        fake_network_allocation_multi_updated = dict(
            fake_network_allocation_multi)
        fake_network_allocation_multi_updated.update(
            network_allocation_update_data)
        fake_share_network_multi_updated = dict(fake_share_network_multi)
        fake_share_network_multi_updated.update(network_update_data)
        fake_share_network_multi_updated.update(fake_neutron_subnet)
        config_data = {
            'DEFAULT': {
                'neutron_net_id': 'fake net id',
                'neutron_subnet_id': 'fake subnet id',
                'neutron_physical_net_name': 'net1',
            }
        }
        self.bind_plugin = self._get_neutron_network_plugin_instance(
            config_data)
        self.bind_plugin.db = db_api

        self.mock_object(self.bind_plugin, '_has_provider_network_extension')
        self.bind_plugin._has_provider_network_extension.return_value = True
        self.mock_object(self.bind_plugin, '_wait_for_ports_bind')
        neutron_host_id_opts = plugin.neutron_bind_network_plugin_opts[1]
        self.mock_object(neutron_host_id_opts, 'default')
        neutron_host_id_opts.default = 'foohost1'

        self.mock_object(db_api, 'network_allocation_create')
        db_api.network_allocation_create.return_value = (
            fake_network_allocation_multi)
        self.mock_object(db_api, 'network_allocation_update')
        db_api.network_allocation_update.return_value = (
            fake_network_allocation_multi_updated)
        self.mock_object(self.bind_plugin.neutron_api, 'get_network')
        self.bind_plugin.neutron_api.get_network.return_value = (
            fake_neutron_network_multi)
        self.mock_object(self.bind_plugin.neutron_api, 'get_subnet')
        self.bind_plugin.neutron_api.get_subnet.return_value = (
            fake_neutron_subnet)
        self.mock_object(db_api, 'share_network_subnet_update')

        with mock.patch.object(self.bind_plugin.neutron_api, 'create_port',
                               mock.Mock(return_value=fake_neutron_port)):
            self.bind_plugin.allocate_network(
                self.fake_context,
                fake_share_server,
                fake_share_network,
                self.fake_share_network_multi,
                allocation_info={'count': 1})

            self.bind_plugin._has_provider_network_extension.assert_any_call()
            expected_kwargs = {
                'binding:vnic_type': 'baremetal',
                'host_id': 'foohost1',
                'network_id': fake_share_network_multi['neutron_net_id'],
                'subnet_id': fake_share_network_multi['neutron_subnet_id'],
                'device_owner': 'manila:share',
                'device_id': fake_share_network_multi['id']
            }
            self.bind_plugin.neutron_api.create_port.assert_called_once_with(
                fake_share_network_multi['project_id'], **expected_kwargs)
            db_api.network_allocation_create.assert_called_once_with(
                self.fake_context,
                fake_network_allocation_multi)
            db_api.share_network_subnet_update.assert_called_with(
                self.fake_context,
                fake_share_network_multi['id'],
                network_update_data)
            network_allocation_update_data['cidr'] = (
                fake_neutron_subnet['cidr'])
            network_allocation_update_data['ip_version'] = (
                fake_neutron_subnet['ip_version'])
            db_api.network_allocation_update.assert_called_once_with(
                self.fake_context,
                fake_neutron_port['id'],
                network_allocation_update_data)

    @ddt.data({
        'neutron_binding_profiles': None,
        'binding_profiles': {}
    }, {
        'neutron_binding_profiles': 'fake_profile',
        'binding_profiles': {}
    }, {
        'neutron_binding_profiles': 'fake_profile',
        'binding_profiles': None
    }, {
        'neutron_binding_profiles': 'fake_profile',
        'binding_profiles': {
            'fake_profile': {
                'neutron_switch_id': 'fake switch id',
                'neutron_port_id': 'fake port id',
                'neutron_switch_info': 'switch_ip: 127.0.0.1'
            }
        }
    }, {
        'neutron_binding_profiles': None,
        'binding_profiles': {
            'fake_profile': {
                'neutron_switch_id': 'fake switch id',
                'neutron_port_id': 'fake port id',
                'neutron_switch_info': 'switch_ip: 127.0.0.1'
            }
        }
    }, {
        'neutron_binding_profiles': 'fake_profile_one,fake_profile_two',
        'binding_profiles': {
            'fake_profile_one': {
                'neutron_switch_id': 'fake switch id 1',
                'neutron_port_id': 'fake port id 1',
                'neutron_switch_info': 'switch_ip: 127.0.0.1'
            },
            'fake_profile_two': {
                'neutron_switch_id': 'fake switch id 2',
                'neutron_port_id': 'fake port id 2',
                'neutron_switch_info': 'switch_ip: 127.0.0.2'
            }
        }
    }, {
        'neutron_binding_profiles': 'fake_profile_two',
        'binding_profiles': {
            'fake_profile_one': {
                'neutron_switch_id': 'fake switch id 1',
                'neutron_port_id': 'fake port id 1',
                'neutron_switch_info': 'switch_ip: 127.0.0.1'
            },
            'fake_profile_two': {
                'neutron_switch_id': 'fake switch id 2',
                'neutron_port_id': 'fake port id 2',
                'neutron_switch_info': 'switch_ip: 127.0.0.2'
            }
        }
    })
    @ddt.unpack
    @mock.patch.object(db_api, 'share_network_get',
                       mock.Mock(return_value=fake_share_network))
    @mock.patch.object(db_api, 'share_server_get',
                       mock.Mock(return_value=fake_share_server))
    def test__get_port_create_args(self, neutron_binding_profiles,
                                   binding_profiles):
        fake_device_owner = 'share'
        fake_host_id = 'fake host'
        neutron_host_id_opts = plugin.neutron_bind_network_plugin_opts[1]
        self.mock_object(neutron_host_id_opts, 'default')
        neutron_host_id_opts.default = fake_host_id

        config_data = {
            'DEFAULT': {
                'neutron_net_id': fake_neutron_network['id'],
                'neutron_subnet_id': fake_neutron_network['subnets'][0]
            }
        }
        # Simulate absence of set values
        if neutron_binding_profiles:
            config_data['DEFAULT'][
                'neutron_binding_profiles'] = neutron_binding_profiles
        if binding_profiles:
            for name, binding_profile in binding_profiles.items():
                config_data[name] = binding_profile

        instance = self._get_neutron_network_plugin_instance(config_data)

        create_args = instance._get_port_create_args(fake_share_server,
                                                     fake_share_network_subnet,
                                                     fake_device_owner)

        expected_create_args = {
            'binding:vnic_type': 'baremetal',
            'host_id': fake_host_id,
            'network_id': fake_share_network_subnet['neutron_net_id'],
            'subnet_id': fake_share_network_subnet['neutron_subnet_id'],
            'device_owner': 'manila:' + fake_device_owner,
            'device_id': fake_share_server['id']
        }
        if neutron_binding_profiles:
            expected_create_args['binding:profile'] = {
                'local_link_information': []
            }
            local_links = expected_create_args[
                'binding:profile']['local_link_information']
            for profile in neutron_binding_profiles.split(','):
                if binding_profiles is None:
                    binding_profile = {}
                else:
                    binding_profile = binding_profiles.get(profile, {})
                local_links.append({
                    'port_id': binding_profile.get('neutron_port_id', None),
                    'switch_id': binding_profile.get('neutron_switch_id', None)
                })
                switch_info = binding_profile.get('neutron_switch_info', None)
                if switch_info is None:
                    local_links[-1]['switch_info'] = None
                else:
                    local_links[-1]['switch_info'] = cfg.types.Dict()(
                        switch_info)

        self.assertEqual(expected_create_args, create_args)

    @mock.patch.object(db_api, 'share_network_get',
                       mock.Mock(return_value=fake_share_network))
    @mock.patch.object(db_api, 'share_server_get',
                       mock.Mock(return_value=fake_share_server))
    def test__get_port_create_args_host_id(self):
        fake_device_owner = 'share'
        fake_host_id = 'fake host'

        config_data = {
            'DEFAULT': {
                'neutron_net_id': fake_neutron_network['id'],
                'neutron_subnet_id': fake_neutron_network['subnets'][0],
                'neutron_host_id': fake_host_id
            }
        }

        instance = self._get_neutron_network_plugin_instance(config_data)

        create_args = instance._get_port_create_args(fake_share_server,
                                                     fake_share_network_subnet,
                                                     fake_device_owner)

        expected_create_args = {
            'binding:vnic_type': 'baremetal',
            'host_id': fake_host_id,
            'network_id': fake_share_network_subnet['neutron_net_id'],
            'subnet_id': fake_share_network_subnet['neutron_subnet_id'],
            'device_owner': 'manila:' + fake_device_owner,
            'device_id': fake_share_server['id']
        }

        self.assertEqual(expected_create_args, create_args)


@ddt.ddt
class NeutronBindSingleNetworkPluginTest(test.TestCase):
    def setUp(self):
        super(NeutronBindSingleNetworkPluginTest, self).setUp()
        self.context = 'fake_context'
        self.fake_context = context.RequestContext(user_id='fake user',
                                                   project_id='fake project',
                                                   is_admin=False)
        self.has_binding_ext_mock = self.mock_object(
            neutron_api.API, '_has_port_binding_extension')
        self.has_binding_ext_mock.return_value = True
        self.bind_plugin = plugin.NeutronBindNetworkPlugin()
        self.bind_plugin.db = db_api
        self.sleep_mock = self.mock_object(time, 'sleep')
        self.bind_plugin = self._get_neutron_network_plugin_instance()
        self.bind_plugin.db = db_api

    def _get_neutron_network_plugin_instance(self, config_data=None):
        if not config_data:
            fake_net_id = 'fake net id'
            fake_subnet_id = 'fake subnet id'
            config_data = {
                'DEFAULT': {
                    'neutron_net_id': fake_net_id,
                    'neutron_subnet_id': fake_subnet_id,
                    'neutron_physical_net_name': 'net1',
                }
            }
            fake_net = {'subnets': ['fake1', 'fake2', fake_subnet_id]}
            self.mock_object(
                neutron_api.API, 'get_network',
                mock.Mock(return_value=fake_net))
        with test_utils.create_temp_config_with_opts(config_data):
            return plugin.NeutronBindSingleNetworkPlugin()

    def test_allocate_network(self):
        self.mock_object(plugin.NeutronNetworkPlugin, 'allocate_network')
        plugin.NeutronNetworkPlugin.allocate_network.return_value = [
            'port1', 'port2']
        instance = self._get_neutron_network_plugin_instance()
        share_server = 'fake_share_server'
        share_network = {}
        share_network_subnet = {'neutron_net_id': {}}
        share_network_upd = {'neutron_net_id': {'upd': True}}
        count = 2
        device_owner = 'fake_device_owner'
        self.mock_object(
            instance, '_update_share_network_net_data',
            mock.Mock(return_value=share_network_upd))
        self.mock_object(instance, '_wait_for_ports_bind', mock.Mock())

        instance.allocate_network(
            self.context, share_server, share_network, share_network_subnet,
            count=count, device_owner=device_owner)

        instance._update_share_network_net_data.assert_called_once_with(
            self.context, share_network_subnet)
        plugin.NeutronNetworkPlugin.allocate_network.assert_called_once_with(
            self.context, share_server, share_network, share_network_upd,
            count=count, device_owner=device_owner)
        instance._wait_for_ports_bind.assert_called_once_with(
            ['port1', 'port2'], share_server)

    def test_init_valid(self):
        fake_net_id = 'fake_net_id'
        fake_subnet_id = 'fake_subnet_id'
        config_data = {
            'DEFAULT': {
                'neutron_net_id': fake_net_id,
                'neutron_subnet_id': fake_subnet_id,
            }
        }
        fake_net = {'subnets': ['fake1', 'fake2', fake_subnet_id]}
        self.mock_object(
            neutron_api.API, 'get_network', mock.Mock(return_value=fake_net))

        with test_utils.create_temp_config_with_opts(config_data):
            instance = plugin.NeutronSingleNetworkPlugin()

            self.assertEqual(fake_net_id, instance.net)
            self.assertEqual(fake_subnet_id, instance.subnet)
            neutron_api.API.get_network.assert_called_once_with(fake_net_id)

    @ddt.data(
        {'net': None, 'subnet': None},
        {'net': 'fake_net_id', 'subnet': None},
        {'net': None, 'subnet': 'fake_subnet_id'})
    @ddt.unpack
    def test_init_invalid(self, net, subnet):
        config_data = dict()
        # Simulate absence of set values
        if net:
            config_data['neutron_net_id'] = net
        if subnet:
            config_data['neutron_subnet_id'] = subnet
        config_data = dict(DEFAULT=config_data)

        with test_utils.create_temp_config_with_opts(config_data):
            self.assertRaises(
                exception.NetworkBadConfigurationException,
                plugin.NeutronSingleNetworkPlugin)

    @ddt.data({}, {'subnets': []}, {'subnets': ['different_foo_subnet']})
    def test_init_subnet_does_not_belong_to_net(self, fake_net):
        fake_net_id = 'fake_net_id'
        config_data = {
            'DEFAULT': {
                'neutron_net_id': fake_net_id,
                'neutron_subnet_id': 'fake_subnet_id',
            }
        }
        self.mock_object(
            neutron_api.API, 'get_network', mock.Mock(return_value=fake_net))

        with test_utils.create_temp_config_with_opts(config_data):
            self.assertRaises(
                exception.NetworkBadConfigurationException,
                plugin.NeutronSingleNetworkPlugin)
            neutron_api.API.get_network.assert_called_once_with(fake_net_id)

    def _get_neutron_single_network_plugin_instance(self):
        fake_subnet_id = 'fake_subnet_id'
        config_data = {
            'DEFAULT': {
                'neutron_net_id': 'fake_net_id',
                'neutron_subnet_id': fake_subnet_id,
            }
        }
        fake_net = {'subnets': [fake_subnet_id]}
        self.mock_object(
            neutron_api.API, 'get_network', mock.Mock(return_value=fake_net))
        with test_utils.create_temp_config_with_opts(config_data):
            instance = plugin.NeutronSingleNetworkPlugin()
            return instance

    def test___update_share_network_net_data_same_values(self):
        instance = self._get_neutron_single_network_plugin_instance()
        share_network = {
            'neutron_net_id': instance.net,
            'neutron_subnet_id': instance.subnet,
        }

        result = instance._update_share_network_net_data(
            self.context, share_network)

        self.assertEqual(share_network, result)

    def test___update_share_network_net_data_different_values_empty(self):
        instance = self._get_neutron_single_network_plugin_instance()
        share_network_subnet_input = {
            'id': 'fake_share_network_id',
        }
        share_network_result = {
            'neutron_net_id': instance.net,
            'neutron_subnet_id': instance.subnet,
        }
        self.mock_object(
            instance.db, 'share_network_subnet_update',
            mock.Mock(return_value='foo'))

        instance._update_share_network_net_data(
            self.context, share_network_subnet_input)

        instance.db.share_network_subnet_update.assert_called_once_with(
            self.context, share_network_subnet_input['id'],
            share_network_result)

    @ddt.data(
        {'n': 'fake_net_id', 's': 'bar'},
        {'n': 'foo', 's': 'fake_subnet_id'})
    @ddt.unpack
    def test___update_share_network_net_data_different_values(self, n, s):
        instance = self._get_neutron_single_network_plugin_instance()
        share_network = {
            'id': 'fake_share_network_id',
            'neutron_net_id': n,
            'neutron_subnet_id': s,
        }
        self.mock_object(
            instance.db, 'share_network_update',
            mock.Mock(return_value=share_network))

        self.assertRaises(
            exception.NetworkBadConfigurationException,
            instance._update_share_network_net_data,
            self.context, share_network)
        self.assertFalse(instance.db.share_network_update.called)

    def test_wait_for_bind(self):
        self.mock_object(self.bind_plugin.neutron_api, 'show_port')
        self.bind_plugin.neutron_api.show_port.return_value = fake_neutron_port

        self.bind_plugin._wait_for_ports_bind([fake_neutron_port],
                                              fake_share_server)

        self.bind_plugin.neutron_api.show_port.assert_called_once_with(
            fake_neutron_port['id'])
        self.sleep_mock.assert_not_called()

    def test_wait_for_bind_error(self):
        fake_neut_port = copy.copy(fake_neutron_port)
        fake_neut_port['status'] = 'ERROR'
        self.mock_object(self.bind_plugin.neutron_api, 'show_port')
        self.bind_plugin.neutron_api.show_port.return_value = fake_neut_port

        self.assertRaises(exception.NetworkException,
                          self.bind_plugin._wait_for_ports_bind,
                          [fake_neut_port, fake_neut_port],
                          fake_share_server)

        self.bind_plugin.neutron_api.show_port.assert_called_once_with(
            fake_neutron_port['id'])
        self.sleep_mock.assert_not_called()

    @ddt.data(('DOWN', 'ACTIVE'), ('DOWN', 'DOWN'), ('ACTIVE', 'DOWN'))
    def test_wait_for_bind_two_ports_no_bind(self, state):
        fake_neut_port1 = copy.copy(fake_neutron_port)
        fake_neut_port1['status'] = state[0]
        fake_neut_port2 = copy.copy(fake_neutron_port)
        fake_neut_port2['status'] = state[1]
        self.mock_object(self.bind_plugin.neutron_api, 'show_port')
        self.bind_plugin.neutron_api.show_port.side_effect = (
            [fake_neut_port1, fake_neut_port2] * 20)

        self.assertRaises(exception.NetworkBindException,
                          self.bind_plugin._wait_for_ports_bind,
                          [fake_neut_port1, fake_neut_port2],
                          fake_share_server)

    @mock.patch.object(db_api, 'network_allocation_create',
                       mock.Mock(return_values=fake_network_allocation))
    @mock.patch.object(db_api, 'share_network_get',
                       mock.Mock(return_value=fake_share_network))
    @mock.patch.object(db_api, 'share_server_get',
                       mock.Mock(return_value=fake_share_server))
    def test_allocate_network_one_allocation(self):
        self.mock_object(self.bind_plugin, '_has_provider_network_extension')
        self.bind_plugin._has_provider_network_extension.return_value = True
        save_nw_data = self.mock_object(self.bind_plugin,
                                        '_save_neutron_network_data')
        save_subnet_data = self.mock_object(self.bind_plugin,
                                            '_save_neutron_subnet_data')
        self.mock_object(self.bind_plugin, '_wait_for_ports_bind')
        neutron_host_id_opts = plugin.neutron_bind_network_plugin_opts[1]
        self.mock_object(neutron_host_id_opts, 'default')
        neutron_host_id_opts.default = 'foohost1'
        self.mock_object(db_api, 'network_allocation_create')

        with mock.patch.object(self.bind_plugin.neutron_api, 'create_port',
                               mock.Mock(return_value=fake_neutron_port)):
            self.bind_plugin.allocate_network(
                self.fake_context,
                fake_share_server,
                fake_share_network,
                fake_share_network_subnet,
                allocation_info={'count': 1})

            self.bind_plugin._has_provider_network_extension.assert_any_call()
            save_nw_data.assert_called_once_with(self.fake_context,
                                                 fake_share_network_subnet)
            save_subnet_data.assert_called_once_with(self.fake_context,
                                                     fake_share_network_subnet)
            expected_kwargs = {
                'binding:vnic_type': 'baremetal',
                'host_id': 'foohost1',
                'network_id': fake_share_network_subnet['neutron_net_id'],
                'subnet_id': fake_share_network_subnet['neutron_subnet_id'],
                'device_owner': 'manila:share',
                'device_id': fake_share_network['id'],
            }
            self.bind_plugin.neutron_api.create_port.assert_called_once_with(
                fake_share_network['project_id'], **expected_kwargs)
            db_api.network_allocation_create.assert_called_once_with(
                self.fake_context,
                fake_network_allocation)
            self.bind_plugin._wait_for_ports_bind.assert_called_once_with(
                [db_api.network_allocation_create(
                    self.fake_context, fake_network_allocation)],
                fake_share_server)

    @ddt.data({
        'neutron_binding_profiles': None,
        'binding_profiles': {}
    }, {
        'neutron_binding_profiles': 'fake_profile',
        'binding_profiles': {}
    }, {
        'neutron_binding_profiles': 'fake_profile',
        'binding_profiles': None
    }, {
        'neutron_binding_profiles': 'fake_profile',
        'binding_profiles': {
            'fake_profile': {
                'neutron_switch_id': 'fake switch id',
                'neutron_port_id': 'fake port id',
                'neutron_switch_info': 'switch_ip: 127.0.0.1'
            }
        }
    }, {
        'neutron_binding_profiles': None,
        'binding_profiles': {
            'fake_profile': {
                'neutron_switch_id': 'fake switch id',
                'neutron_port_id': 'fake port id',
                'neutron_switch_info': 'switch_ip: 127.0.0.1'
            }
        }
    }, {
        'neutron_binding_profiles': 'fake_profile_one,fake_profile_two',
        'binding_profiles': {
            'fake_profile_one': {
                'neutron_switch_id': 'fake switch id 1',
                'neutron_port_id': 'fake port id 1',
                'neutron_switch_info': 'switch_ip: 127.0.0.1'
            },
            'fake_profile_two': {
                'neutron_switch_id': 'fake switch id 2',
                'neutron_port_id': 'fake port id 2',
                'neutron_switch_info': 'switch_ip: 127.0.0.2'
            }
        }
    }, {
        'neutron_binding_profiles': 'fake_profile_two',
        'binding_profiles': {
            'fake_profile_one': {
                'neutron_switch_id': 'fake switch id 1',
                'neutron_port_id': 'fake port id 1',
                'neutron_switch_info': 'switch_ip: 127.0.0.1'
            },
            'fake_profile_two': {
                'neutron_switch_id': 'fake switch id 2',
                'neutron_port_id': 'fake port id 2',
                'neutron_switch_info': 'switch_ip: 127.0.0.2'
            }
        }
    })
    @ddt.unpack
    @mock.patch.object(db_api, 'share_network_get',
                       mock.Mock(return_value=fake_share_network))
    @mock.patch.object(db_api, 'share_server_get',
                       mock.Mock(return_value=fake_share_server))
    def test__get_port_create_args(self, neutron_binding_profiles,
                                   binding_profiles):
        fake_device_owner = 'share'
        fake_host_id = 'fake host'
        neutron_host_id_opts = plugin.neutron_bind_network_plugin_opts[1]
        self.mock_object(neutron_host_id_opts, 'default')
        neutron_host_id_opts.default = fake_host_id

        config_data = {
            'DEFAULT': {
                'neutron_net_id': fake_neutron_network['id'],
                'neutron_subnet_id': fake_neutron_network['subnets'][0]
            }
        }
        # Simulate absence of set values
        if neutron_binding_profiles:
            config_data['DEFAULT'][
                'neutron_binding_profiles'] = neutron_binding_profiles
        if binding_profiles:
            for name, binding_profile in binding_profiles.items():
                config_data[name] = binding_profile

        instance = self._get_neutron_network_plugin_instance(config_data)

        create_args = instance._get_port_create_args(fake_share_server,
                                                     fake_share_network_subnet,
                                                     fake_device_owner)

        expected_create_args = {
            'binding:vnic_type': 'baremetal',
            'host_id': fake_host_id,
            'network_id': fake_share_network_subnet['neutron_net_id'],
            'subnet_id': fake_share_network_subnet['neutron_subnet_id'],
            'device_owner': 'manila:' + fake_device_owner,
            'device_id': fake_share_server['id']
        }
        if neutron_binding_profiles:
            expected_create_args['binding:profile'] = {
                'local_link_information': []
            }
            local_links = expected_create_args[
                'binding:profile']['local_link_information']
            for profile in neutron_binding_profiles.split(','):
                if binding_profiles is None:
                    binding_profile = {}
                else:
                    binding_profile = binding_profiles.get(profile, {})
                local_links.append({
                    'port_id': binding_profile.get('neutron_port_id', None),
                    'switch_id': binding_profile.get('neutron_switch_id', None)
                })
                switch_info = binding_profile.get('neutron_switch_info', None)
                if switch_info is None:
                    local_links[-1]['switch_info'] = None
                else:
                    local_links[-1]['switch_info'] = cfg.types.Dict()(
                        switch_info)

        self.assertEqual(expected_create_args, create_args)

    @mock.patch.object(db_api, 'share_network_get',
                       mock.Mock(return_value=fake_share_network))
    @mock.patch.object(db_api, 'share_server_get',
                       mock.Mock(return_value=fake_share_server))
    def test__get_port_create_args_host_id(self):
        fake_device_owner = 'share'
        fake_host_id = 'fake host'

        config_data = {
            'DEFAULT': {
                'neutron_net_id': fake_neutron_network['id'],
                'neutron_subnet_id': fake_neutron_network['subnets'][0],
                'neutron_host_id': fake_host_id
            }
        }

        instance = self._get_neutron_network_plugin_instance(config_data)

        create_args = instance._get_port_create_args(fake_share_server,
                                                     fake_share_network_subnet,
                                                     fake_device_owner)

        expected_create_args = {
            'binding:vnic_type': 'baremetal',
            'host_id': fake_host_id,
            'network_id': fake_share_network_subnet['neutron_net_id'],
            'subnet_id': fake_share_network_subnet['neutron_subnet_id'],
            'device_owner': 'manila:' + fake_device_owner,
            'device_id': fake_share_server['id']
        }

        self.assertEqual(expected_create_args, create_args)


class NeutronBindNetworkPluginWithNormalTypeTest(test.TestCase):
    def setUp(self):
        super(NeutronBindNetworkPluginWithNormalTypeTest, self).setUp()
        config_data = {
            'DEFAULT': {
                'neutron_vnic_type': 'normal',
            }
        }
        self.plugin = plugin.NeutronNetworkPlugin()
        self.plugin.db = db_api
        self.fake_context = context.RequestContext(user_id='fake user',
                                                   project_id='fake project',
                                                   is_admin=False)

        with test_utils.create_temp_config_with_opts(config_data):
            self.bind_plugin = plugin.NeutronBindNetworkPlugin()
            self.bind_plugin.db = db_api

    @mock.patch.object(db_api, 'network_allocation_create',
                       mock.Mock(return_values=fake_network_allocation))
    @mock.patch.object(db_api, 'share_network_get',
                       mock.Mock(return_value=fake_share_network))
    @mock.patch.object(db_api, 'share_server_get',
                       mock.Mock(return_value=fake_share_server))
    def test_allocate_network_one_allocation(self):
        self.mock_object(self.bind_plugin, '_has_provider_network_extension')
        self.bind_plugin._has_provider_network_extension.return_value = True
        save_nw_data = self.mock_object(self.bind_plugin,
                                        '_save_neutron_network_data')
        save_subnet_data = self.mock_object(self.bind_plugin,
                                            '_save_neutron_subnet_data')
        self.mock_object(self.bind_plugin, '_wait_for_ports_bind')
        neutron_host_id_opts = plugin.neutron_bind_network_plugin_opts[1]
        self.mock_object(neutron_host_id_opts, 'default')
        neutron_host_id_opts.default = 'foohost1'
        self.mock_object(db_api, 'network_allocation_create')
        multi_seg = self.mock_object(
            self.bind_plugin, '_is_neutron_multi_segment')
        multi_seg.return_value = False

        with mock.patch.object(self.bind_plugin.neutron_api, 'create_port',
                               mock.Mock(return_value=fake_neutron_port)):
            self.bind_plugin.allocate_network(
                self.fake_context,
                fake_share_server,
                fake_share_network,
                fake_share_network_subnet,
                allocation_info={'count': 1})

            self.bind_plugin._has_provider_network_extension.assert_any_call()
            save_nw_data.assert_called_once_with(self.fake_context,
                                                 fake_share_network_subnet)
            save_subnet_data.assert_called_once_with(self.fake_context,
                                                     fake_share_network_subnet)
            expected_kwargs = {
                'binding:vnic_type': 'normal',
                'host_id': 'foohost1',
                'network_id': fake_share_network_subnet['neutron_net_id'],
                'subnet_id': fake_share_network_subnet['neutron_subnet_id'],
                'device_owner': 'manila:share',
                'device_id': fake_share_server['id'],
            }
            self.bind_plugin.neutron_api.create_port.assert_called_once_with(
                fake_share_network['project_id'], **expected_kwargs)
            db_api.network_allocation_create.assert_called_once_with(
                self.fake_context,
                fake_network_allocation)
            self.bind_plugin._wait_for_ports_bind.assert_not_called()

    def test_update_network_allocation(self):
        self.mock_object(self.bind_plugin, '_wait_for_ports_bind')
        self.mock_object(db_api, 'network_allocations_get_for_share_server')
        db_api.network_allocations_get_for_share_server.return_value = [
            fake_neutron_port]

        self.bind_plugin.update_network_allocation(self.fake_context,
                                                   fake_share_server)

        self.bind_plugin._wait_for_ports_bind.assert_called_once_with(
            [fake_neutron_port], fake_share_server)


@ddt.ddt
class NeutronBindSingleNetworkPluginWithNormalTypeTest(test.TestCase):
    def setUp(self):
        super(NeutronBindSingleNetworkPluginWithNormalTypeTest, self).setUp()
        fake_net_id = 'fake net id'
        fake_subnet_id = 'fake subnet id'
        config_data = {
            'DEFAULT': {
                'neutron_net_id': fake_net_id,
                'neutron_subnet_id': fake_subnet_id,
                'neutron_vnic_type': 'normal',
            }
        }
        fake_net = {'subnets': ['fake1', 'fake2', fake_subnet_id]}
        self.mock_object(
            neutron_api.API, 'get_network', mock.Mock(return_value=fake_net))
        self.plugin = plugin.NeutronNetworkPlugin()
        self.plugin.db = db_api
        self.fake_context = context.RequestContext(user_id='fake user',
                                                   project_id='fake project',
                                                   is_admin=False)

        with test_utils.create_temp_config_with_opts(config_data):
            self.bind_plugin = plugin.NeutronBindSingleNetworkPlugin()
            self.bind_plugin.db = db_api

    @mock.patch.object(db_api, 'network_allocation_create',
                       mock.Mock(return_values=fake_network_allocation))
    @mock.patch.object(db_api, 'share_network_get',
                       mock.Mock(return_value=fake_share_network))
    @mock.patch.object(db_api, 'share_server_get',
                       mock.Mock(return_value=fake_share_server))
    def test_allocate_network_one_allocation(self):
        self.mock_object(self.bind_plugin, '_has_provider_network_extension')
        self.bind_plugin._has_provider_network_extension.return_value = True
        save_nw_data = self.mock_object(self.bind_plugin,
                                        '_save_neutron_network_data')
        save_subnet_data = self.mock_object(self.bind_plugin,
                                            '_save_neutron_subnet_data')
        self.mock_object(self.bind_plugin, '_wait_for_ports_bind')
        neutron_host_id_opts = plugin.neutron_bind_network_plugin_opts[1]
        self.mock_object(neutron_host_id_opts, 'default')
        neutron_host_id_opts.default = 'foohost1'
        self.mock_object(db_api, 'network_allocation_create')

        with mock.patch.object(self.bind_plugin.neutron_api, 'create_port',
                               mock.Mock(return_value=fake_neutron_port)):
            self.bind_plugin.allocate_network(
                self.fake_context,
                fake_share_server,
                fake_share_network,
                fake_share_network_subnet,
                allocation_info={'count': 1})

            self.bind_plugin._has_provider_network_extension.assert_any_call()
            save_nw_data.assert_called_once_with(self.fake_context,
                                                 fake_share_network_subnet)
            save_subnet_data.assert_called_once_with(self.fake_context,
                                                     fake_share_network_subnet)
            expected_kwargs = {
                'binding:vnic_type': 'normal',
                'host_id': 'foohost1',
                'network_id': fake_share_network_subnet['neutron_net_id'],
                'subnet_id': fake_share_network_subnet['neutron_subnet_id'],
                'device_owner': 'manila:share',
                'device_id': fake_share_network['id'],
            }
            self.bind_plugin.neutron_api.create_port.assert_called_once_with(
                fake_share_network['project_id'], **expected_kwargs)
            db_api.network_allocation_create.assert_called_once_with(
                self.fake_context,
                fake_network_allocation)
            self.bind_plugin._wait_for_ports_bind.assert_not_called()

    def test_update_network_allocation(self):
        self.mock_object(self.bind_plugin, '_wait_for_ports_bind')
        self.mock_object(db_api, 'network_allocations_get_for_share_server')
        db_api.network_allocations_get_for_share_server.return_value = [
            fake_neutron_port]

        self.bind_plugin.update_network_allocation(self.fake_context,
                                                   fake_share_server)

        self.bind_plugin._wait_for_ports_bind.assert_called_once_with(
            [fake_neutron_port], fake_share_server)

    @ddt.data({'fix_ips': [{'ip_address': 'test_ip'},
                           {'ip_address': '10.78.223.129'}],
               'ip_version': 4},
              {'fix_ips': [{'ip_address': 'test_ip'},
                           {'ip_address': 'ad80::abaa:0:c2:2'}],
               'ip_version': 6},
              {'fix_ips': [{'ip_address': '10.78.223.129'},
                           {'ip_address': 'ad80::abaa:0:c2:2'}],
               'ip_version': 6},
              )
    @ddt.unpack
    def test__get_matched_ip_address(self, fix_ips, ip_version):
        result = self.bind_plugin._get_matched_ip_address(fix_ips, ip_version)
        self.assertEqual(fix_ips[1]['ip_address'], result)

    @ddt.data({'fix_ips': [{'ip_address': 'test_ip_1'},
                           {'ip_address': 'test_ip_2'}],
               'ip_version': (4, 6)},
              {'fix_ips': [{'ip_address': 'ad80::abaa:0:c2:1'},
                           {'ip_address': 'ad80::abaa:0:c2:2'}],
               'ip_version': (4, )},
              {'fix_ips': [{'ip_address': '192.0.0.2'},
                           {'ip_address': '192.0.0.3'}],
               'ip_version': (6, )},
              {'fix_ips': [{'ip_address': '192.0.0.2/12'},
                           {'ip_address': '192.0.0.330'},
                           {'ip_address': 'ad80::001::ad80'},
                           {'ip_address': 'ad80::abaa:0:c2:2/64'}],
               'ip_version': (4, 6)},
              )
    @ddt.unpack
    def test__get_matched_ip_address_illegal(self, fix_ips, ip_version):
        for version in ip_version:
            self.assertRaises(exception.NetworkBadConfigurationException,
                              self.bind_plugin._get_matched_ip_address,
                              fix_ips, version)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""add provider_location to share_snapshot_instances

Revision ID: eb6d5544cbbd
Revises: 5155c7077f99
Create Date: 2016-02-12 22:25:39.594545

"""

# revision identifiers, used by Alembic.
revision = 'eb6d5544cbbd'
down_revision = '5155c7077f99'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column(
        'share_snapshot_instances',
        sa.Column('provider_location', sa.String(255), nullable=True))


def downgrade():
    op.drop_column('share_snapshot_instances', 'provider_location')



# Copyright (c) 2016 Mirantis, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import re

from oslo_config import cfg
from oslo_log import log

from manila import exception
from manila.i18n import _
from manila.share import driver
from manila.share import utils as share_utils

CONF = cfg.CONF

lv_opts = [
    cfg.StrOpt("container_volume_group",
               default="manila_docker_volumes",
               help="LVM volume group to use for volumes. This volume group "
                    "must be created by the cloud administrator independently "
                    "from manila operations."),
]

CONF.register_opts(lv_opts)
LOG = log.getLogger(__name__)


class LVMHelper(driver.ExecuteMixin):

    def __init__(self, *args, **kwargs):
        self.configuration = kwargs.pop("configuration", None)
        if self.configuration is None:
            raise exception.ManilaException(_("LVMHelper called without "
                                              "supplying configuration."))
        self.configuration.append_config_values(lv_opts)
        super(LVMHelper, self).__init__(*args, **kwargs)
        self.init_execute_mixin()

    def get_share_server_pools(self, share_server=None):
        out, err = self._execute('vgs',
                                 self.configuration.container_volume_group,
                                 '--options', 'vg_size,vg_free',
                                 '--noheadings',
                                 '--units', 'g',
                                 run_as_root=True)
        if err:
            msg = _("Unable to gather size of the volume group %(vg)s to be "
                    "used by the driver. Error: %(err)s")
            raise exception.ShareBackendException(
                msg % {'vg': self.configuration.container_volume_group,
                       'err': err})

        (free_size, total_size) = sorted(re.findall(r"\d+\.\d+|\d+", out),
                                         reverse=False)
        return [{
            'pool_name': self.configuration.container_volume_group,
            'total_capacity_gb': float(total_size),
            'free_capacity_gb': float(free_size),
            'reserved_percentage': 0,
            'reserved_snapshot_percentage': 0,
        }, ]

    def _get_lv_device(self, share_name):
        return os.path.join("/dev", self.configuration.container_volume_group,
                            share_name)

    def _get_lv_folder(self, share_name):
        return os.path.join(self.configuration.container_volume_mount_path,
                            share_name)

    def provide_storage(self, share_name, size):
        self._execute("lvcreate", "-p", "rw", "-L",
                      str(size) + "G", "-n", share_name,
                      self.configuration.container_volume_group,
                      run_as_root=True)
        self._execute("mkfs.ext4", self._get_lv_device(share_name),
                      run_as_root=True)

    def _try_to_unmount_device(self, device):
        # NOTE(ganso): We invoke this method to be sure volume was unmounted,
        # and we swallow the exception in case it fails to.
        try:
            self._execute("umount", device, run_as_root=True)
        except exception.ProcessExecutionError as e:
            LOG.warning("Failed to umount helper directory %(device)s due to "
                        "%(reason)s.", {'device': device, 'reason': e})

    def remove_storage(self, share_name):
        device = self._get_lv_device(share_name)
        self._try_to_unmount_device(device)

        # (aovchinnikov): bug 1621784 manifests itself in jamming logical
        # volumes, so try removing once and issue warning until it is fixed.
        try:
            self._execute("lvremove", "-f", "--autobackup", "n",
                          device, run_as_root=True)
        except exception.ProcessExecutionError as e:
            LOG.warning("Failed to remove logical volume %(device)s due to "
                        "%(reason)s.", {'device': device, 'reason': e})

    def rename_storage(self, share_name, new_share_name):
        old_device = self._get_lv_device(share_name)
        new_device = self._get_lv_device(new_share_name)

        self._try_to_unmount_device(old_device)

        try:
            self._execute("lvrename", "--autobackup", "n",
                          old_device, new_device, run_as_root=True)
        except exception.ProcessExecutionError as e:
            msg = ("Failed to rename logical volume %(device)s due to "
                   "%(reason)s." % {'device': old_device, 'reason': e})
            LOG.exception(msg)
            raise

    def extend_share(self, share_name, new_size, share_server=None):
        lv_device = self._get_lv_device(share_name)
        cmd = ('lvextend', '-L', '%sG' % new_size, '-n', lv_device)
        self._execute(*cmd, run_as_root=True)
        self._execute("e2fsck", "-f", "-y", lv_device, run_as_root=True)
        self._execute('resize2fs', lv_device, run_as_root=True)

    def get_size(self, share_name):
        device = self._get_lv_device(share_name)
        size = self._execute(
            "lvs", "-o", "lv_size", "--noheadings", "--nosuffix",
            "--units", "g", device, run_as_root=True)
        LOG.debug("Found size %(size)s for LVM device "
                  "%(lvm)s.", {'size': size[0], 'lvm': share_name})
        return size[0]

    def migration_check_compatibility(self, context, source_share,
                                      destination_share, share_server=None,
                                      destination_share_server=None):
        """Checks compatibility between self.host and destination host."""
        # They must be in same vg and host
        compatible = False
        destination_host = destination_share['host']
        source_host = source_share['host']
        destination_vg = share_utils.extract_host(
            destination_host, level='pool')
        source_vg = share_utils.extract_host(
            source_host, level='pool')

        if destination_vg != source_vg:
            msg = ("Cannot migrate share %(shr)s between "
                   "%(src)s and %(dest)s, they must be in the same volume "
                   "group.")
            msg_args = {
                'shr': source_share['id'],
                'src': source_share['host'],
                'dest': destination_host,
            }
            LOG.exception(msg, msg_args)
        else:
            compatible = True

        compatibility = {
            'compatible': compatible,
            'writable': True,
            'nondisruptive': False,
            'preserve_metadata': True,
            'preserve_snapshots': False,
        }

        return compatibility

    def migration_start(self, context, source_share, destination_share,
                        source_snapshots, snapshot_mappings,
                        share_server=None, destination_share_server=None):
        """Starts the migration of the share from one host to another."""

        # NOTE(felipe_rodrigues): Since they are in the same volume group,
        # there is no need to copy the data between the volumes.
        return

    def migration_continue(self, context, source_share, destination_share,
                           source_snapshots, snapshot_mappings,
                           share_server=None, destination_share_server=None):
        """Check the progress of the migration."""
        return True

    def migration_get_progress(self, context, source_share,
                               destination_share, source_snapshots,
                               snapshot_mappings, share_server=None,
                               destination_share_server=None):
        """Return detailed progress of the migration in progress."""
        return {
            'total_progress': 100,
        }

    def migration_cancel(self, context, source_share, destination_share,
                         source_snapshots, snapshot_mappings,
                         share_server=None, destination_share_server=None):
        """Abort an ongoing migration."""

        # NOTE(felipe_rodrigues): Since they are in the same volume group,
        # there is no need to cancel the copy of the data.
        return

    def migration_complete(self, context, source_share, destination_share,
                           source_snapshots, snapshot_mappings,
                           share_server=None, destination_share_server=None):
        """Completes by removing the source local volume."""

        # NOTE(felipe_rodrigues): Since they are in the same volume group,
        # there is no need to remove source lv.
        return

    def share_server_migration_check_compatibility(
            self, context, share_server, dest_host, old_share_network,
            new_share_network, shares_request_spec):
        """Is called to check migration compatibility for a share server."""
        not_compatible = {
            'compatible': False,
            'writable': None,
            'nondisruptive': None,
            'preserve_snapshots': None,
            'migration_cancel': None,
            'migration_get_progress': None,
        }

        dest_backend_name = share_utils.extract_host(dest_host,
                                                     level='backend_name')
        source_backend_name = share_utils.extract_host(share_server['host'],
                                                       level='backend_name')
        if dest_backend_name == source_backend_name:
            msg = _("Cannot perform server migration %(server)s within the "
                    "same backend. Please choose a destination host different "
                    "from the source.")
            msg_args = {
                'server': share_server['id'],
            }
            LOG.error(msg, msg_args)
            return not_compatible

        # The container backend has only one pool, gets its pool name from the
        # first instance.
        first_share = shares_request_spec['shares_req_spec'][0]
        source_host = first_share['share_instance_properties']['host']
        source_vg = share_utils.extract_host(
            source_host, level='pool')
        dest_vg = share_utils.extract_host(
            dest_host, level='pool')
        if dest_vg and dest_vg != source_vg:
            msg = ("Cannot migrate share server %(server)s between %(src)s "
                   "and %(dest)s. They must be in the same volume group.")
            msg_args = {
                'server': share_server['id'],
                'src': source_host,
                'dest': dest_host,
            }
            LOG.error(msg, msg_args)
            return not_compatible

        # NOTE(felipe_rodrigues): it is not required to check the capacity,
        # because it is migrating in the same volume group.

        return {
            'compatible': True,
            'writable': True,
            'nondisruptive': False,
            'preserve_snapshots': False,
            'migration_cancel': True,
            'migration_get_progress': True,
        }

    def share_server_migration_start(self, context, src_share_server,
                                     dest_share_server, shares, snapshots):
        """Is called to perform 1st phase of migration of a share server."""

        # NOTE(felipe_rodrigues): Since they are in the same volume group,
        # there is no need to copy the data between the volumes.
        return

    def share_server_migration_continue(self, context, src_share_server,
                                        dest_share_server, shares, snapshots):
        """Check the progress of the migration."""
        return True

    def share_server_migration_complete(self, context, source_share_server,
                                        dest_share_server, shares, snapshots,
                                        new_network_allocations):
        """Completes by removing the source local volume."""

        # NOTE(felipe_rodrigues): Since they are in the same volume group,
        # there is no need to remove source lv.
        return

    def share_server_migration_cancel(self, context, src_share_server,
                                      dest_share_server, shares, snapshots):
        """Abort an ongoing migration."""

        # NOTE(felipe_rodrigues): Since they are in the same volume group,
        # there is no need to cancel the copy of the data.
        return

    def share_server_migration_get_progress(self, context, src_share_server,
                                            dest_share_server, shares,
                                            snapshots):
        """Return detailed progress of the server migration in progress."""

        return {
            'total_progress': 100,
        }

    def get_share_pool_name(self, share_id):
        """Return the pool name where the share is allocated"""

        return self.configuration.container_volume_group



# Copyright (c) Goutham Pacha Ravi.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""Unit Tests for the interface methods in the manila/db/api.py."""

import re

from manila.db import api as db_interface
from manila.db.sqlalchemy import api as db_api
from manila import test


class DBInterfaceTestCase(test.TestCase):
    """Test cases for the DB Interface methods."""

    def test_interface_methods(self):
        """Ensure that implementation methods match interfaces.

        manila/db/api module is merely shim layer between the database
        implementation and the other methods using these implementations.
        Bugs are introduced when the shims go out of sync with the actual
        implementation. So this test ensures that method names and
        signatures match between the interface and the implementation.
        """
        members = dir(db_interface)
        # Ignore private methods for the file and any other members that
        # need not match.
        ignore_members = re.compile(r'^_|CONF|IMPL')
        interfaces = [i for i in members if not ignore_members.match(i)]
        for interface in interfaces:
            method = getattr(db_interface, interface)
            if callable(method):
                mock_method_call = self.mock_object(db_api, interface)
                # kwargs always specify defaults, ignore them in the signature.
                args = filter(
                    lambda x: x != 'kwargs', method.__code__.co_varnames)

                method(*args)

                self.assertTrue(mock_method_call.called)



# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# Copyright 2010 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Utility methods for working with WSGI servers."""

import webob.dec
import webob.exc

from manila.i18n import _


class Request(webob.Request):
    pass


class Application(object):
    """Base WSGI application wrapper. Subclasses need to implement __call__."""

    @classmethod
    def factory(cls, global_config, **local_config):
        """Used for paste app factories in paste.deploy config files.

        Any local configuration (that is, values under the [app:APPNAME]
        section of the paste config) will be passed into the `__init__` method
        as kwargs.

        A hypothetical configuration would look like:

            [app:wadl]
            latest_version = 1.3
            paste.app_factory = manila.api.fancy_api:Wadl.factory

        which would result in a call to the `Wadl` class as

            import manila.api.fancy_api
            fancy_api.Wadl(latest_version='1.3')

        You could of course re-implement the `factory` method in subclasses,
        but using the kwarg passing it shouldn't be necessary.

        """
        return cls(**local_config)

    def __call__(self, environ, start_response):
        r"""Subclasses will probably want to implement __call__ like this:

        @webob.dec.wsgify(RequestClass=Request)
        def __call__(self, req):
          # Any of the following objects work as responses:

          # Option 1: simple string
          res = 'message\n'

          # Option 2: a nicely formatted HTTP exception page
          res = exc.HTTPForbidden(detail='Nice try')

          # Option 3: a webob Response object (in case you need to play with
          # headers, or you want to be treated like an iterable, or or or)
          res = Response();
          res.app_iter = open('somefile')

          # Option 4: any wsgi app to be run next
          res = self.application

          # Option 5: you can get a Response object for a wsgi app, too, to
          # play with headers etc
          res = req.get_response(self.application)

          # You can then just return your response...
          return res
          # ... or set req.response and return None.
          req.response = res

        See the end of http://pythonpaste.org/webob/modules/dec.html
        for more info.

        """
        raise NotImplementedError(_('You must implement __call__'))


class Middleware(Application):
    """Base WSGI middleware.

    These classes require an application to be
    initialized that will be called next.  By default the middleware will
    simply call its wrapped app, or you can override __call__ to customize its
    behavior.

    """

    @classmethod
    def factory(cls, global_config, **local_config):
        """Used for paste app factories in paste.deploy config files.

        Any local configuration (that is, values under the [filter:APPNAME]
        section of the paste config) will be passed into the `__init__` method
        as kwargs.

        A hypothetical configuration would look like:

            [filter:analytics]
            redis_host = 127.0.0.1
            paste.filter_factory = manila.api.analytics:Analytics.factory

        which would result in a call to the `Analytics` class as

            import manila.api.analytics
            analytics.Analytics(app_from_paste, redis_host='127.0.0.1')

        You could of course re-implement the `factory` method in subclasses,
        but using the kwarg passing it shouldn't be necessary.

        """
        def _factory(app):
            return cls(app, **local_config)
        return _factory

    def __init__(self, application):
        self.application = application

    def process_request(self, req):
        """Called on each request.

        If this returns None, the next application down the stack will be
        executed. If it returns a response then that response will be returned
        and execution will stop here.

        """
        return None

    def process_response(self, response):
        """Do whatever you'd like to the response."""
        return response

    @webob.dec.wsgify(RequestClass=Request)
    def __call__(self, req):
        # pylint: disable=assignment-from-none
        response = self.process_request(req)
        if response:
            return response
        response = req.get_response(self.application)
        return self.process_response(response)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""The messages API controller module.

This module handles the following requests:
GET /messages
GET /messages/<message_id>
DELETE /messages/<message_id>
"""

from oslo_utils import timeutils
from six.moves import http_client
import webob
from webob import exc

from manila.api import common
from manila.api.openstack import wsgi
from manila.api.views import messages as messages_view
from manila import exception
from manila.i18n import _
from manila.message import api as message_api

MESSAGES_BASE_MICRO_VERSION = '2.37'
MESSAGES_QUERY_BY_TIMESTAMP = '2.52'


class MessagesController(wsgi.Controller):
    """The User Messages API controller for the OpenStack API."""

    _view_builder_class = messages_view.ViewBuilder
    resource_name = 'message'

    def __init__(self):
        self.message_api = message_api.API()
        super(MessagesController, self).__init__()

    @wsgi.Controller.api_version(MESSAGES_BASE_MICRO_VERSION)
    @wsgi.Controller.authorize('get')
    def show(self, req, id):
        """Return the given message."""
        context = req.environ['manila.context']

        try:
            message = self.message_api.get(context, id)
        except exception.MessageNotFound as error:
            raise exc.HTTPNotFound(explanation=error.msg)

        return self._view_builder.detail(req, message)

    @wsgi.Controller.api_version(MESSAGES_BASE_MICRO_VERSION)
    @wsgi.Controller.authorize
    @wsgi.action("delete")
    def delete(self, req, id):
        """Delete a message."""
        context = req.environ['manila.context']

        try:
            message = self.message_api.get(context, id)
            self.message_api.delete(context, message)
        except exception.MessageNotFound as error:
            raise exc.HTTPNotFound(explanation=error.msg)

        return webob.Response(status_int=http_client.NO_CONTENT)

    @wsgi.Controller.api_version(MESSAGES_BASE_MICRO_VERSION, '2.51')
    @wsgi.Controller.authorize('get_all')
    def index(self, req):
        """Returns a list of messages, transformed through view builder."""
        context = req.environ['manila.context']
        filters = req.params.copy()

        params = common.get_pagination_params(req)
        limit, offset = [params.get('limit'), params.get('offset')]
        sort_key, sort_dir = common.get_sort_params(filters)
        filters.pop('created_since', None)
        filters.pop('created_before', None)

        messages = self.message_api.get_all(context, search_opts=filters,
                                            limit=limit,
                                            offset=offset,
                                            sort_key=sort_key,
                                            sort_dir=sort_dir)

        return self._view_builder.index(req, messages)

    @wsgi.Controller.api_version(MESSAGES_QUERY_BY_TIMESTAMP)   # noqa: F811
    @wsgi.Controller.authorize('get_all')
    def index(self, req):  # pylint: disable=function-redefined  # noqa F811
        """Returns a list of messages, transformed through view builder."""
        context = req.environ['manila.context']
        filters = req.params.copy()

        params = common.get_pagination_params(req)
        limit, offset = [params.get('limit'), params.get('offset')]
        sort_key, sort_dir = common.get_sort_params(filters)

        for time_comparison_filter in ['created_since', 'created_before']:
            if time_comparison_filter in filters:
                time_str = filters.get(time_comparison_filter)
                try:
                    parsed_time = timeutils.parse_isotime(time_str)
                except ValueError:
                    msg = _('Invalid value specified for the query '
                            'key: %s') % time_comparison_filter
                    raise exc.HTTPBadRequest(explanation=msg)

                filters[time_comparison_filter] = parsed_time

        messages = self.message_api.get_all(context, search_opts=filters,
                                            limit=limit,
                                            offset=offset,
                                            sort_key=sort_key,
                                            sort_dir=sort_dir)

        return self._view_builder.index(req, messages)


def create_resource():
    return wsgi.Resource(MessagesController())



# Copyright 2015 Alex Meade
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log
from oslo_utils import uuidutils
import six
from six.moves import http_client
import webob
from webob import exc

from manila.api import common
from manila.api.openstack import api_version_request as api_version
from manila.api.openstack import wsgi
from manila.api.views import share_groups as share_group_views
from manila import db
from manila import exception
from manila.i18n import _
from manila.share import share_types
from manila.share_group import api as share_group_api
from manila.share_group import share_group_types


LOG = log.getLogger(__name__)
SG_GRADUATION_VERSION = '2.55'


class ShareGroupController(wsgi.Controller, wsgi.AdminActionsMixin):
    """The Share Groups API controller for the OpenStack API."""

    resource_name = 'share_group'
    _view_builder_class = share_group_views.ShareGroupViewBuilder

    def __init__(self):
        super(ShareGroupController, self).__init__()
        self.share_group_api = share_group_api.API()

    def _get_share_group(self, context, share_group_id):
        try:
            return self.share_group_api.get(context, share_group_id)
        except exception.NotFound:
            msg = _("Share group %s not found.") % share_group_id
            raise exc.HTTPNotFound(explanation=msg)

    @wsgi.Controller.authorize('get')
    def _show(self, req, id):
        """Return data about the given share group."""
        context = req.environ['manila.context']
        share_group = self._get_share_group(context, id)
        return self._view_builder.detail(req, share_group)

    @wsgi.Controller.api_version('2.31', '2.54', experimental=True)
    def show(self, req, id):
        return self._show(req, id)

    @wsgi.Controller.api_version(SG_GRADUATION_VERSION)  # noqa
    def show(self, req, id):  # pylint: disable=function-redefined  # noqa F811
        return self._show(req, id)

    @wsgi.Controller.authorize('delete')
    def _delete_share_group(self, req, id):
        """Delete a share group."""
        context = req.environ['manila.context']

        LOG.info("Delete share group with id: %s", id, context=context)
        share_group = self._get_share_group(context, id)
        try:
            self.share_group_api.delete(context, share_group)
        except exception.InvalidShareGroup as e:
            raise exc.HTTPConflict(explanation=six.text_type(e))
        return webob.Response(status_int=http_client.ACCEPTED)

    @wsgi.Controller.api_version('2.31', '2.54', experimental=True)
    def delete(self, req, id):
        return self._delete_share_group(req, id)

    @wsgi.Controller.api_version(SG_GRADUATION_VERSION)  # noqa
    def delete(self, req, id):  # pylint: disable=function-redefined  # noqa F811
        return self._delete_share_group(req, id)

    @wsgi.Controller.api_version('2.31', '2.54', experimental=True)
    def index(self, req):
        return self._get_share_groups(req, is_detail=False)

    @wsgi.Controller.api_version(SG_GRADUATION_VERSION)  # noqa
    def index(self, req):  # pylint: disable=function-redefined  # noqa F811
        return self._get_share_groups(req, is_detail=False)

    @wsgi.Controller.api_version('2.31', '2.54', experimental=True)
    def detail(self, req):
        return self._get_share_groups(req, is_detail=True)

    @wsgi.Controller.api_version(SG_GRADUATION_VERSION)  # noqa
    def detail(self, req):  # pylint: disable=function-redefined  # noqa F811
        return self._get_share_groups(req, is_detail=True)

    @wsgi.Controller.authorize('get_all')
    def _get_share_groups(self, req, is_detail):
        """Returns a summary or detail list of share groups."""
        context = req.environ['manila.context']

        search_opts = {}
        search_opts.update(req.GET)

        # Remove keys that are not related to share group attrs
        search_opts.pop('limit', None)
        search_opts.pop('offset', None)
        sort_key = search_opts.pop('sort_key', 'created_at')
        sort_dir = search_opts.pop('sort_dir', 'desc')
        if req.api_version_request < api_version.APIVersionRequest("2.36"):
            search_opts.pop('name~', None)
            search_opts.pop('description~', None)
        if 'group_type_id' in search_opts:
            search_opts['share_group_type_id'] = search_opts.pop(
                'group_type_id')

        share_groups = self.share_group_api.get_all(
            context, detailed=is_detail, search_opts=search_opts,
            sort_dir=sort_dir, sort_key=sort_key,
        )

        limited_list = common.limited(share_groups, req)

        if is_detail:
            share_groups = self._view_builder.detail_list(req, limited_list)
        else:
            share_groups = self._view_builder.summary_list(req, limited_list)
        return share_groups

    @wsgi.Controller.authorize('update')
    def _update_share_group(self, req, id, body):
        """Update a share group."""
        context = req.environ['manila.context']

        if not self.is_valid_body(body, 'share_group'):
            msg = _("'share_group' is missing from the request body.")
            raise exc.HTTPBadRequest(explanation=msg)

        share_group_data = body['share_group']
        valid_update_keys = {'name', 'description'}
        invalid_fields = set(share_group_data.keys()) - valid_update_keys
        if invalid_fields:
            msg = _("The fields %s are invalid or not allowed to be updated.")
            raise exc.HTTPBadRequest(explanation=msg % invalid_fields)

        share_group = self._get_share_group(context, id)
        share_group = self.share_group_api.update(
            context, share_group, share_group_data)
        return self._view_builder.detail(req, share_group)

    @wsgi.Controller.api_version('2.31', '2.54', experimental=True)
    def update(self, req, id, body):
        return self._update_share_group(req, id, body)

    @wsgi.Controller.api_version(SG_GRADUATION_VERSION)  # noqa
    def update(self, req, id, body):  # pylint: disable=function-redefined  # noqa F811
        return self._update_share_group(req, id, body)

    @wsgi.Controller.authorize('create')
    def _create(self, req, body):
        """Creates a new share group."""
        context = req.environ['manila.context']

        if not self.is_valid_body(body, 'share_group'):
            msg = _("'share_group' is missing from the request body.")
            raise exc.HTTPBadRequest(explanation=msg)

        share_group = body['share_group']
        valid_fields = {
            'name',
            'description',
            'share_types',
            'share_group_type_id',
            'source_share_group_snapshot_id',
            'share_network_id',
            'availability_zone',
        }
        invalid_fields = set(share_group.keys()) - valid_fields
        if invalid_fields:
            msg = _("The fields %s are invalid.") % invalid_fields
            raise exc.HTTPBadRequest(explanation=msg)

        if ('share_types' in share_group and
                'source_share_group_snapshot_id' in share_group):
            msg = _("Cannot supply both 'share_types' and "
                    "'source_share_group_snapshot_id' attributes.")
            raise exc.HTTPBadRequest(explanation=msg)

        if not (share_group.get('share_types') or
                'source_share_group_snapshot_id' in share_group):
            default_share_type = share_types.get_default_share_type()
            if default_share_type:
                share_group['share_types'] = [default_share_type['id']]
            else:
                msg = _("Must specify at least one share type as a default "
                        "share type has not been configured.")
                raise exc.HTTPBadRequest(explanation=msg)

        kwargs = {}

        if 'name' in share_group:
            kwargs['name'] = share_group.get('name')
        if 'description' in share_group:
            kwargs['description'] = share_group.get('description')

        _share_types = share_group.get('share_types')
        if _share_types:
            if not all([uuidutils.is_uuid_like(st) for st in _share_types]):
                msg = _("The 'share_types' attribute must be a list of uuids")
                raise exc.HTTPBadRequest(explanation=msg)
            kwargs['share_type_ids'] = _share_types

        if ('share_network_id' in share_group and
                'source_share_group_snapshot_id' in share_group):
            msg = _("Cannot supply both 'share_network_id' and "
                    "'source_share_group_snapshot_id' attributes as the share "
                    "network is inherited from the source.")
            raise exc.HTTPBadRequest(explanation=msg)

        availability_zone = share_group.get('availability_zone')
        if availability_zone:
            if 'source_share_group_snapshot_id' in share_group:
                msg = _(
                    "Cannot supply both 'availability_zone' and "
                    "'source_share_group_snapshot_id' attributes as the "
                    "availability zone is inherited from the source.")
                raise exc.HTTPBadRequest(explanation=msg)
            try:
                az = db.availability_zone_get(context, availability_zone)
                kwargs['availability_zone_id'] = az.id
                kwargs['availability_zone'] = az.name
            except exception.AvailabilityZoneNotFound as e:
                raise exc.HTTPNotFound(explanation=six.text_type(e))

        if 'source_share_group_snapshot_id' in share_group:
            source_share_group_snapshot_id = share_group.get(
                'source_share_group_snapshot_id')
            if not uuidutils.is_uuid_like(source_share_group_snapshot_id):
                msg = _("The 'source_share_group_snapshot_id' attribute "
                        "must be a uuid.")
                raise exc.HTTPBadRequest(explanation=six.text_type(msg))
            kwargs['source_share_group_snapshot_id'] = (
                source_share_group_snapshot_id)
        elif 'share_network_id' in share_group:
            share_network_id = share_group.get('share_network_id')
            if not uuidutils.is_uuid_like(share_network_id):
                msg = _("The 'share_network_id' attribute must be a uuid.")
                raise exc.HTTPBadRequest(explanation=six.text_type(msg))
            kwargs['share_network_id'] = share_network_id

        if 'share_group_type_id' in share_group:
            share_group_type_id = share_group.get('share_group_type_id')
            if not uuidutils.is_uuid_like(share_group_type_id):
                msg = _("The 'share_group_type_id' attribute must be a uuid.")
                raise exc.HTTPBadRequest(explanation=six.text_type(msg))
            kwargs['share_group_type_id'] = share_group_type_id
        else:  # get default
            def_share_group_type = share_group_types.get_default()
            if def_share_group_type:
                kwargs['share_group_type_id'] = def_share_group_type['id']
            else:
                msg = _("Must specify a share group type as a default "
                        "share group type has not been configured.")
                raise exc.HTTPBadRequest(explanation=msg)

        try:
            new_share_group = self.share_group_api.create(context, **kwargs)
        except exception.InvalidShareGroupSnapshot as e:
            raise exc.HTTPConflict(explanation=six.text_type(e))
        except (exception.ShareGroupSnapshotNotFound,
                exception.InvalidInput) as e:
            raise exc.HTTPBadRequest(explanation=six.text_type(e))

        return self._view_builder.detail(
            req, {k: v for k, v in new_share_group.items()})

    @wsgi.Controller.api_version('2.31', '2.54', experimental=True)
    @wsgi.response(202)
    def create(self, req, body):
        return self._create(req, body)

    @wsgi.Controller.api_version(SG_GRADUATION_VERSION)  # noqa
    @wsgi.response(202)
    def create(self, req, body):  # pylint: disable=function-redefined  # noqa F811
        return self._create(req, body)

    def _update(self, *args, **kwargs):
        db.share_group_update(*args, **kwargs)

    def _get(self, *args, **kwargs):
        return self.share_group_api.get(*args, **kwargs)

    def _delete(self, context, resource, force=True):
        # Delete all share group snapshots
        for snap in resource['snapshots']:
            db.share_group_snapshot_destroy(context, snap['id'])

        # Delete all shares in share group
        for share in db.get_all_shares_by_share_group(context, resource['id']):
            db.share_delete(context, share['id'])

        db.share_group_destroy(context.elevated(), resource['id'])

    @wsgi.Controller.api_version('2.31', '2.54', experimental=True)
    @wsgi.action('reset_status')
    def share_group_reset_status(self, req, id, body):
        return self._reset_status(req, id, body)

    # pylint: disable=function-redefined
    @wsgi.Controller.api_version(SG_GRADUATION_VERSION)  # noqa
    @wsgi.action('reset_status')
    def share_group_reset_status(self, req, id, body):  # noqa F811
        return self._reset_status(req, id, body)

    # pylint: enable=function-redefined
    @wsgi.Controller.api_version('2.31', '2.54', experimental=True)
    @wsgi.action('force_delete')
    def share_group_force_delete(self, req, id, body):
        return self._force_delete(req, id, body)

    # pylint: disable=function-redefined
    @wsgi.Controller.api_version(SG_GRADUATION_VERSION)  # noqa
    @wsgi.action('force_delete')
    def share_group_force_delete(self, req, id, body):  # noqa F811
        return self._force_delete(req, id, body)


def create_resource():
    return wsgi.Resource(ShareGroupController())



# Copyright (c) 2021 SAP.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from unittest import mock

from manila import exception
from manila.scheduler.filters import affinity
from manila import test
from manila.tests.scheduler import fakes


fake_hosts = [
    fakes.FakeHostState('host1', {}),
    fakes.FakeHostState('host2', {}),
    fakes.FakeHostState('host3', {}),
    ]

fake_shares_1 = {
    'abb6e0ac-7c3e-4ce0-8a69-5a166d246882': {
        'instances': [
            {'host': fake_hosts[0].host}
            ]
        },
    '4de0cc74-450c-4468-8159-52128cf03407': {
        'instances': [
            {'host': fake_hosts[0].host}
            ]
        },
    }

fake_shares_2 = {
    'c920fb61-e250-4c3c-a25d-1fdd9ca7cbc3': {
        'instances': [
            {'host': fake_hosts[1].host}
            ]
        },
    }

fake_shares_3 = {
    '3923bebf-9825-4a66-971e-6092a9fe2dbb': {
        'instances': [
            {'host': fake_hosts[2].host}
            ]
        },
    }


@ddt.ddt
class AffinityFilterTestCase(test.TestCase):
    """Test case for AffinityFilter."""

    def setUp(self):
        super(AffinityFilterTestCase, self).setUp()
        self.filter = affinity.AffinityFilter()
        self.anti_filter = affinity.AntiAffinityFilter()

    def _make_filter_hints(self, *hints):
        return {
            'context': None,
            'scheduler_hints': {'same_host': ','.join(list(hints))},
        }

    def _make_anti_filter_hints(self, *hints):
        return {
            'context': None,
            'scheduler_hints': {'different_host': ','.join(list(hints))},
        }

    def _fake_get(self, context, uuid):
        if uuid in fake_shares_1.keys():
            return fake_shares_1[uuid]
        if uuid in fake_shares_2.keys():
            return fake_shares_2[uuid]
        if uuid in fake_shares_3.keys():
            return fake_shares_3[uuid]
        raise exception.ShareNotFound(uuid)

    @ddt.data('b5c207da-ac0b-43b0-8691-c6c9e860199d')
    @mock.patch('manila.share.api.API.get')
    def test_affinity_share_not_found(self, unknown_id, mock_share_get):
        mock_share_get.side_effect = self._fake_get
        self.assertRaises(exception.ShareNotFound,
                          self.filter._validate,
                          self._make_filter_hints(unknown_id))

    @ddt.data(
        {'context': None},
        {'context': None, 'scheduler_hints': None},
        {'context': None, 'scheduler_hints': {}},
    )
    def test_affinity_scheduler_hint_not_set(self, hints):
        self.assertRaises(affinity.SchedulerHintsNotSet,
                          self.filter._validate, hints)

    @ mock.patch('manila.share.api.API.get')
    def test_affinity_filter(self, mock_share_get):
        mock_share_get.side_effect = self._fake_get

        share_ids = fake_shares_1.keys()
        hints = self._make_filter_hints(*share_ids)
        valid_hosts = self.filter.filter_all(fake_hosts, hints)
        valid_hosts = [h.host for h in valid_hosts]

        self.assertIn('host1', valid_hosts)
        self.assertNotIn('host2', valid_hosts)
        self.assertNotIn('host3', valid_hosts)

    @ mock.patch('manila.share.api.API.get')
    def test_anti_affinity_filter(self, mock_share_get):
        mock_share_get.side_effect = self._fake_get

        share_ids = fake_shares_2.keys()
        hints = self._make_anti_filter_hints(*share_ids)
        valid_hosts = self.anti_filter.filter_all(fake_hosts, hints)
        valid_hosts = [h.host for h in valid_hosts]

        self.assertIn('host1', valid_hosts)
        self.assertIn('host3', valid_hosts)
        self.assertNotIn('host2', valid_hosts)



# Copyright 2015 Hewlett Packard Enterprise Development LP
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
CIFS = 'CIFS'
SMB_LOWER = 'smb'
NFS = 'NFS'
NFS_LOWER = 'nfs'
IP = 'ip'
USER = 'user'
USERNAME = 'USERNAME_0'
ADD_USERNAME = '+USERNAME_0:fullcontrol'
DROP_USERNAME = '-USERNAME_0:fullcontrol'
PASSWORD = 'PASSWORD_0'
READ_WRITE = 'rw'
READ_ONLY = 'ro'
SAN_LOGIN = 'testlogin4san'
SAN_PASSWORD = 'testpassword4san'
API_URL = 'https://1.2.3.4:8080/api/v1'
TIMEOUT = 60
PORT = 22
SHARE_TYPE_ID = 123456789
CIDR_PREFIX = '24'

# Constants to use with Mock and expect in results
EXPECTED_IP_10203040 = '10.20.30.40'
EXPECTED_IP_10203041 = '10.20.30.41'
EXPECTED_IP_1234 = '1.2.3.4'
EXPECTED_MY_IP = '9.8.7.6'
EXPECTED_IP_127 = '127.0.0.1'
EXPECTED_IP_127_2 = '127.0.0.2'
EXPECTED_ACCESS_LEVEL = 'foo_access'
EXPECTED_SUBNET = '255.255.255.0'  # based on CIDR_PREFIX above
EXPECTED_VLAN_TYPE = 'vlan'
EXPECTED_VXLAN_TYPE = 'vxlan'
EXPECTED_VLAN_TAG = '101'
EXPECTED_SERVER_ID = '1a1a1a1a-2b2b-3c3c-4d4d-5e5e5e5e5e5e'
EXPECTED_PROJECT_ID = 'osf-nfs-project-id'
SHARE_ID = 'share-id'
EXPECTED_SHARE_ID = 'osf-share-id'
EXPECTED_SHARE_ID_RO = 'osf-ro-share-id'
EXPECTED_SHARE_NAME = 'share-name'
EXPECTED_NET_NAME = 'testnet'
EXPECTED_FPG = 'pool'
EXPECTED_HOST = 'hostname@backend#' + EXPECTED_FPG
UNEXPECTED_FPG = 'not_a_pool'
UNEXPECTED_HOST = 'hostname@backend#' + UNEXPECTED_FPG
HOST_WITHOUT_POOL_1 = 'hostname@backend'
HOST_WITHOUT_POOL_2 = 'hostname@backend#'
EXPECTED_SHARE_PATH = '/anyfpg/anyvfs/anyfstore'
EXPECTED_SIZE_1 = 1
EXPECTED_SIZE_2 = 2
EXPECTED_SNAP_NAME = 'osf-snap-name'
EXPECTED_SNAP_ID = 'osf-snap-id'
EXPECTED_STATS = {'test': 'stats'}
EXPECTED_FPG_CONF = [{EXPECTED_FPG: [EXPECTED_IP_10203040]}]
EXPECTED_FSTORE = EXPECTED_PROJECT_ID
EXPECTED_VFS = 'test_vfs'
EXPECTED_GET_VFS = {'vfsname': EXPECTED_VFS,
                    'vfsip': {'address': [EXPECTED_IP_10203040]}}
EXPECTED_GET_VFS_MULTIPLES = {
    'vfsname': EXPECTED_VFS,
    'vfsip': {'address': [EXPECTED_IP_10203041, EXPECTED_IP_10203040]}}

EXPECTED_CLIENT_GET_VFS_MEMBERS_MULTI = {
    'fspname': EXPECTED_VFS,
    'vfsip': [
        {'networkName': EXPECTED_NET_NAME,
         'fspool': EXPECTED_VFS,
         'address': EXPECTED_IP_10203040,
         'prefixLen': EXPECTED_SUBNET,
         'vfs': EXPECTED_VFS,
         'vlanTag': EXPECTED_VLAN_TAG,
         },
        {'networkName': EXPECTED_NET_NAME,
         'fspool': EXPECTED_VFS,
         'address': EXPECTED_IP_10203041,
         'prefixLen': EXPECTED_SUBNET,
         'vfs': EXPECTED_VFS,
         'vlanTag': EXPECTED_VLAN_TAG,
         },
    ],
    'vfsname': EXPECTED_VFS,
    }
EXPECTED_MEDIATOR_GET_VFS_RET_VAL_MULTI = {
    'fspname': EXPECTED_VFS,
    'vfsip': {
        'networkName': EXPECTED_NET_NAME,
        'fspool': EXPECTED_VFS,
        'address': [
            EXPECTED_IP_10203040,
            EXPECTED_IP_10203041,
        ],
        'prefixLen': EXPECTED_SUBNET,
        'vfs': EXPECTED_VFS,
        'vlanTag': EXPECTED_VLAN_TAG
    },
    'vfsname': EXPECTED_VFS,
    }

EXPECTED_CLIENT_GET_VFS_MEMBERS = {
    'fspname': EXPECTED_VFS,
    'vfsip': {
        'networkName': EXPECTED_NET_NAME,
        'fspool': EXPECTED_VFS,
        'address': EXPECTED_IP_10203040,
        'prefixLen': EXPECTED_SUBNET,
        'vfs': EXPECTED_VFS,
        'vlanTag': EXPECTED_VLAN_TAG,
    },
    'vfsname': EXPECTED_VFS,
    }
EXPECTED_MEDIATOR_GET_VFS_RET_VAL = {
    'fspname': EXPECTED_VFS,
    'vfsip': {
        'networkName': EXPECTED_NET_NAME,
        'fspool': EXPECTED_VFS,
        'address': [EXPECTED_IP_10203040],
        'prefixLen': EXPECTED_SUBNET,
        'vfs': EXPECTED_VFS,
        'vlanTag': EXPECTED_VLAN_TAG,
    },
    'vfsname': EXPECTED_VFS,
    }
EXPECTED_CLIENT_GET_VFS_RETURN_VALUE = {
    'total': 1,
    'members': [EXPECTED_CLIENT_GET_VFS_MEMBERS],
    }
EXPECTED_CLIENT_GET_VFS_RETURN_VALUE_MULTI = {
    'total': 1,
    'members': [EXPECTED_CLIENT_GET_VFS_MEMBERS_MULTI],
    }
EXPECTED_FPG_MAP = {EXPECTED_FPG: {EXPECTED_VFS: [EXPECTED_IP_10203040]}}
EXPECTED_FPG_MAP_MULTI_VFS = {EXPECTED_FPG: {
    EXPECTED_VFS: [EXPECTED_IP_10203041, EXPECTED_IP_10203040]}}
EXPECTED_SHARE_IP = '10.50.3.8'
EXPECTED_HPE_DEBUG = True
EXPECTED_COMMENT = "OpenStack Manila - foo-comment"
EXPECTED_EXTRA_SPECS = {}
EXPECTED_LOCATION = ':'.join((EXPECTED_IP_1234, EXPECTED_SHARE_PATH))
EXPECTED_SUPER_SHARE = 'OPENSTACK_SUPER_SHARE'
EXPECTED_SUPER_SHARE_COMMENT = ('OpenStack super share used to delete nested '
                                'shares.')
EXPECTED_CIFS_DOMAIN = 'LOCAL_CLUSTER'
EXPECTED_MOUNT_PATH = '/mnt/'

SHARE_SERVER = {
    'backend_details': {
        'ip': EXPECTED_IP_10203040,
        'fpg': EXPECTED_FPG,
        'vfs': EXPECTED_VFS,
        },
}

# Access rules. Allow for overwrites.
ACCESS_RULE_NFS = {
    'access_type': IP,
    'access_to': EXPECTED_IP_1234,
    'access_level': READ_WRITE,
}

ACCESS_RULE_CIFS = {
    'access_type': USER,
    'access_to': USERNAME,
    'access_level': READ_WRITE,
}

ADD_RULE_BAD_TYPE = {
    'access_type': 'unsupported_other_type',
    'access_to': USERNAME,
    'access_level': READ_WRITE,
}

ADD_RULE_IP = {
    'access_type': IP,
    'access_to': EXPECTED_IP_1234,
    'access_level': READ_WRITE,
}

ADD_RULE_IP_RO = {
    'access_type': IP,
    'access_to': EXPECTED_IP_1234,
    'access_level': READ_ONLY,
}

ADD_RULE_USER = {
    'access_type': USER,
    'access_to': USERNAME,
    'access_level': READ_WRITE,
}

DELETE_RULE_IP = {
    'access_type': IP,
    'access_to': EXPECTED_IP_1234,
    'access_level': READ_WRITE,
}

DELETE_RULE_USER = {
    'access_type': USER,
    'access_to': USERNAME,
    'access_level': READ_WRITE,
}

DELETE_RULE_IP_RO = {
    'access_type': IP,
    'access_to': EXPECTED_IP_1234,
    'access_level': READ_ONLY,
}

GET_FSQUOTA = {'message': None,
               'total': 1,
               'members': [{'hardBlock': '1024', 'softBlock': '1024'}]}

EXPECTED_FSIP = {
    'fspool': EXPECTED_FPG,
    'vfs': EXPECTED_VFS,
    'address': EXPECTED_IP_1234,
    'prefixLen': EXPECTED_SUBNET,
    'vlanTag': EXPECTED_VLAN_TAG,
}

OTHER_FSIP = {
    'fspool': EXPECTED_FPG,
    'vfs': EXPECTED_VFS,
    'address': '9.9.9.9',
    'prefixLen': EXPECTED_SUBNET,
    'vlanTag': EXPECTED_VLAN_TAG,
}

NFS_SHARE_INFO = {
    'project_id': EXPECTED_PROJECT_ID,
    'id': EXPECTED_SHARE_ID,
    'share_proto': NFS,
    'export_location': EXPECTED_LOCATION,
    'size': 1234,
    'host': EXPECTED_HOST,
}

SNAPSHOT_INFO = {
    'name': EXPECTED_SNAP_NAME,
    'id': EXPECTED_SNAP_ID,
    'share': {
        'project_id': EXPECTED_PROJECT_ID,
        'id': EXPECTED_SHARE_ID,
        'share_proto': NFS,
        'export_location': EXPECTED_LOCATION,
        'host': EXPECTED_HOST,
    },
}

SNAPSHOT_INSTANCE = {
    'name': EXPECTED_SNAP_NAME,
    'id': EXPECTED_SNAP_ID,
    'share_id': EXPECTED_SHARE_ID,
    'share_proto': NFS,
}


class FakeException(Exception):
    pass


FAKE_EXCEPTION = FakeException("Fake exception for testing.")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""add_security_service_update_control_fields

Revision ID: 478c445d8d3e
Revises: 0c23aec99b74
Create Date: 2020-12-07 12:33:41.444202

"""

# revision identifiers, used by Alembic.
revision = '478c445d8d3e'
down_revision = '0c23aec99b74'

from alembic import op
from manila.common import constants
from oslo_log import log
import sqlalchemy as sa

SHARE_SERVERS_TABLE = 'share_servers'
SHARE_NETWORKS_TABLE = 'share_networks'
ASYNC_OPERATION_DATA_TABLE = 'async_operation_data'
LOG = log.getLogger(__name__)


def upgrade():
    context = op.get_context()
    mysql_dl = context.bind.dialect.name == 'mysql'
    datetime_type = (sa.dialects.mysql.DATETIME(fsp=6)
                     if mysql_dl else sa.DateTime)
    try:
        op.create_table(
            ASYNC_OPERATION_DATA_TABLE,
            sa.Column('created_at', datetime_type),
            sa.Column('updated_at', datetime_type),
            sa.Column('deleted_at', datetime_type),
            sa.Column('deleted', sa.Integer, default=0),
            sa.Column('entity_uuid', sa.String(36),
                      nullable=False, primary_key=True),
            sa.Column('key', sa.String(255),
                      nullable=False, primary_key=True),
            sa.Column('value', sa.String(1023), nullable=False),
            mysql_engine='InnoDB',
        )
        op.add_column(
            SHARE_SERVERS_TABLE,
            sa.Column('security_service_update_support', sa.Boolean,
                      nullable=False, server_default=sa.sql.false())
        )
        op.add_column(
            SHARE_NETWORKS_TABLE,
            sa.Column('status', sa.String(36), nullable=False,
                      server_default=constants.STATUS_NETWORK_ACTIVE))
    except Exception:
        msg_args = {
            'async_op_table': ASYNC_OPERATION_DATA_TABLE,
            'sec_serv_column': 'share_servers.security_service_update_support',
            'shr_net_column': 'share_networks.status',
        }
        LOG.error('Table %(async_op_table)s and table columns '
                  '%(sec_serv_column)s and %(shr_net_column)s were not'
                  ' created!', msg_args)
        raise


def downgrade():
    try:
        op.drop_table(ASYNC_OPERATION_DATA_TABLE)
        op.drop_column(SHARE_SERVERS_TABLE, 'security_service_update_support')
        op.drop_column(SHARE_NETWORKS_TABLE, 'status')
    except Exception:
        msg_args = {
            'async_op_table': ASYNC_OPERATION_DATA_TABLE,
            'sec_serv_column': 'share_servers.security_service_update_support',
            'shr_net_column': 'share_networks.status',
        }
        LOG.error('Table %(async_op_table)s and table columns '
                  '%(sec_serv_column)s and %(shr_net_column)s were not '
                  'dropped!', msg_args)
        raise



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import versionutils
from oslo_policy import policy

from manila.policies import base

BASE_POLICY_NAME = 'share_network:%s'

DEPRECATED_REASON = """
The share network API now support system scope and default roles.
"""

deprecated_share_network_create = policy.DeprecatedRule(
    name=BASE_POLICY_NAME % 'create',
    check_str=base.RULE_DEFAULT,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_share_network_show = policy.DeprecatedRule(
    name=BASE_POLICY_NAME % 'show',
    check_str=base.RULE_DEFAULT,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_share_network_index = policy.DeprecatedRule(
    name=BASE_POLICY_NAME % 'index',
    check_str=base.RULE_DEFAULT,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_share_network_detail = policy.DeprecatedRule(
    name=BASE_POLICY_NAME % 'detail',
    check_str=base.RULE_DEFAULT,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_share_network_update = policy.DeprecatedRule(
    name=BASE_POLICY_NAME % 'update',
    check_str=base.RULE_DEFAULT,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_share_network_delete = policy.DeprecatedRule(
    name=BASE_POLICY_NAME % 'delete',
    check_str=base.RULE_DEFAULT,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_share_network_add_security_service = policy.DeprecatedRule(
    name=BASE_POLICY_NAME % 'add_security_service',
    check_str=base.RULE_DEFAULT,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_share_network_remove_security_service = policy.DeprecatedRule(
    name=BASE_POLICY_NAME % 'remove_security_service',
    check_str=base.RULE_DEFAULT,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_share_network_get_all = policy.DeprecatedRule(
    name=BASE_POLICY_NAME % 'get_all_share_networks',
    check_str=base.RULE_ADMIN_API,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_share_network_add_security_service_check = policy.DeprecatedRule(
    name=BASE_POLICY_NAME % 'add_security_service_check',
    check_str=base.RULE_DEFAULT,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_share_network_update_security_service = policy.DeprecatedRule(
    name=BASE_POLICY_NAME % 'update_security_service',
    check_str=base.RULE_DEFAULT,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_share_network_update_security_service_check = policy.DeprecatedRule(
    name=BASE_POLICY_NAME % 'update_security_service_check',
    check_str=base.RULE_DEFAULT,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_share_network_reset_status = policy.DeprecatedRule(
    name=BASE_POLICY_NAME % 'reset_status',
    check_str=base.RULE_ADMIN_API,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)

share_network_policies = [
    policy.DocumentedRuleDefault(
        name=BASE_POLICY_NAME % 'create',
        check_str=base.SYSTEM_ADMIN_OR_PROJECT_MEMBER,
        scope_types=['system', 'project'],
        description="Create share network.",
        operations=[
            {
                'method': 'POST',
                'path': '/share-networks'
            }
        ],
        deprecated_rule=deprecated_share_network_create
    ),
    policy.DocumentedRuleDefault(
        name=BASE_POLICY_NAME % 'show',
        check_str=base.SYSTEM_OR_PROJECT_READER,
        scope_types=['system', 'project'],
        description="Get details of a share network.",
        operations=[
            {
                'method': 'GET',
                'path': '/share-networks/{share_network_id}'
            }
        ],
        deprecated_rule=deprecated_share_network_show
    ),
    policy.DocumentedRuleDefault(
        name=BASE_POLICY_NAME % 'index',
        check_str=base.SYSTEM_OR_PROJECT_READER,
        scope_types=['system', 'project'],
        description="Get all share networks.",
        operations=[
            {
                'method': 'GET',
                'path': '/share-networks'
            },
            {
                'method': 'GET',
                'path': '/share-networks?{query}'
            }
        ],
        deprecated_rule=deprecated_share_network_index
    ),
    policy.DocumentedRuleDefault(
        name=BASE_POLICY_NAME % 'detail',
        check_str=base.SYSTEM_OR_PROJECT_READER,
        scope_types=['system', 'project'],
        description="Get details of share networks .",
        operations=[
            {
                'method': 'GET',
                'path': '/share-networks/detail?{query}'
            },
            {
                'method': 'GET',
                'path': '/share-networks/detail'
            },
        ],
        deprecated_rule=deprecated_share_network_detail
    ),
    policy.DocumentedRuleDefault(
        name=BASE_POLICY_NAME % 'update',
        check_str=base.SYSTEM_ADMIN_OR_PROJECT_MEMBER,
        scope_types=['system', 'project'],
        description="Update a share network.",
        operations=[
            {
                'method': 'PUT',
                'path': '/share-networks/{share_network_id}'
            }
        ],
        deprecated_rule=deprecated_share_network_update
    ),
    policy.DocumentedRuleDefault(
        name=BASE_POLICY_NAME % 'delete',
        check_str=base.SYSTEM_ADMIN_OR_PROJECT_MEMBER,
        scope_types=['system', 'project'],
        description="Delete a share network.",
        operations=[
            {
                'method': 'DELETE',
                'path': '/share-networks/{share_network_id}'
            }
        ],
        deprecated_rule=deprecated_share_network_delete
    ),
    policy.DocumentedRuleDefault(
        name=BASE_POLICY_NAME % 'add_security_service',
        check_str=base.SYSTEM_ADMIN_OR_PROJECT_MEMBER,
        scope_types=['system', 'project'],
        description="Add security service to share network.",
        operations=[
            {
                'method': 'POST',
                'path': '/share-networks/{share_network_id}/action'
            }
        ],
        deprecated_rule=deprecated_share_network_add_security_service
    ),
    policy.DocumentedRuleDefault(
        name=BASE_POLICY_NAME % 'add_security_service_check',
        check_str=base.SYSTEM_ADMIN_OR_PROJECT_MEMBER,
        scope_types=['system', 'project'],
        description="Check the feasibility of add security service to a share "
                    "network.",
        operations=[
            {
                'method': 'POST',
                'path': '/share-networks/{share_network_id}/action'
            }
        ],
        deprecated_rule=deprecated_share_network_add_security_service_check
    ),
    policy.DocumentedRuleDefault(
        name=BASE_POLICY_NAME % 'remove_security_service',
        check_str=base.SYSTEM_ADMIN_OR_PROJECT_MEMBER,
        scope_types=['system', 'project'],
        description="Remove security service from share network.",
        operations=[
            {
                'method': 'POST',
                'path': '/share-networks/{share_network_id}/action'
            }
        ],
        deprecated_rule=deprecated_share_network_remove_security_service
    ),
    policy.DocumentedRuleDefault(
        name=BASE_POLICY_NAME % 'update_security_service',
        check_str=base.SYSTEM_ADMIN_OR_PROJECT_MEMBER,
        scope_types=['system', 'project'],
        description="Update security service from share network.",
        operations=[
            {
                'method': 'POST',
                'path': '/share-networks/{share_network_id}/action'
            }
        ],
        deprecated_rule=deprecated_share_network_update_security_service
    ),
    policy.DocumentedRuleDefault(
        name=BASE_POLICY_NAME % 'update_security_service_check',
        check_str=base.SYSTEM_ADMIN_OR_PROJECT_MEMBER,
        scope_types=['system', 'project'],
        description="Check the feasibility of update a security service from "
                    "share network.",
        operations=[
            {
                'method': 'POST',
                'path': '/share-networks/{share_network_id}/action'
            }
        ],
        deprecated_rule=deprecated_share_network_update_security_service_check
    ),
    policy.DocumentedRuleDefault(
        name=BASE_POLICY_NAME % 'reset_status',
        check_str=base.SYSTEM_ADMIN_OR_PROJECT_ADMIN,
        scope_types=['system', 'project'],
        description="Reset share network`s status.",
        operations=[
            {
                'method': 'POST',
                'path': '/share-networks/{share_network_id}/action'
            }
        ],
        deprecated_rule=deprecated_share_network_reset_status
    ),
    policy.DocumentedRuleDefault(
        name=BASE_POLICY_NAME % 'get_all_share_networks',
        check_str=base.SYSTEM_READER,
        scope_types=['system'],
        description="Get share networks belonging to all projects.",
        operations=[
            {
                'method': 'GET',
                'path': '/share-networks?all_tenants=1'
            },
            {
                'method': 'GET',
                'path': '/share-networks/detail?all_tenants=1'
            }
        ],
        deprecated_rule=deprecated_share_network_get_all
    ),
]


def list_rules():
    return share_network_policies



# Copyright 2020 NetApp, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Tests for the CreateFromSnapshotFilter.
"""
import ddt

from manila.scheduler.filters import create_from_snapshot
from manila import test
from manila.tests.scheduler import fakes


@ddt.ddt
class CreateFromSnapshotFilterTestCase(test.TestCase):
    """Test case for CreateFromSnapshotFilter."""

    def setUp(self):
        super(CreateFromSnapshotFilterTestCase, self).setUp()
        self.filter = create_from_snapshot.CreateFromSnapshotFilter()

    @staticmethod
    def _create_request(snapshot_id=None,
                        snapshot_host=None,
                        replication_domain=None):
        return {
            'request_spec': {
                'snapshot_id': snapshot_id,
                'snapshot_host': snapshot_host,
            },
            'replication_domain': replication_domain,
        }

    @staticmethod
    def _create_host_state(host=None, rep_domain=None):
        return fakes.FakeHostState(host,
                                   {
                                       'replication_domain': rep_domain,
                                   })

    def test_without_snapshot_id(self):
        request = self._create_request()
        host = self._create_host_state(host='fake_host')

        self.assertTrue(self.filter.host_passes(host, request))

    def test_without_snapshot_host(self):
        request = self._create_request(snapshot_id='fake_snapshot_id',
                                       replication_domain="fake_domain")
        host = self._create_host_state(host='fake_host',
                                       rep_domain='fake_domain_2')

        self.assertTrue(self.filter.host_passes(host, request))

    @ddt.data(('host1@AAA#pool1', 'host1@AAA#pool1'),
              ('host1@AAA#pool1', 'host1@AAA#pool2'))
    @ddt.unpack
    def test_same_backend(self, request_host, host_state):
        request = self._create_request(snapshot_id='fake_snapshot_id',
                                       snapshot_host=request_host)
        host = self._create_host_state(host=host_state)

        self.assertTrue(self.filter.host_passes(host, request))

    def test_same_availability_zone(self):
        request = self._create_request(snapshot_id='fake_snapshot_id',
                                       snapshot_host='fake_host',
                                       replication_domain="fake_domain")
        host = self._create_host_state(host='fake_host_2',
                                       rep_domain='fake_domain')
        self.assertTrue(self.filter.host_passes(host, request))

    def test_different_backend_and_availability_zone(self):
        request = self._create_request(snapshot_id='fake_snapshot_id',
                                       snapshot_host='fake_host',
                                       replication_domain="fake_domain")
        host = self._create_host_state(host='fake_host_2',
                                       rep_domain='fake_domain_2')

        self.assertFalse(self.filter.host_passes(host, request))
